# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_TimeSignature___init___01():
    r'''Initializes time signature from integer pair.
    '''

    time_signature = TimeSignature((9, 32))
    assert isinstance(time_signature, TimeSignature)
    assert format(time_signature, 'lilypond') == '\\time 9/32'


def test_indicatortools_TimeSignature___init___02():
    r'''Initializes time signature from other time signature instance.
    '''

    time_signature_1 = TimeSignature((9, 32))
    time_signature_2 = TimeSignature(time_signature_1)

    assert isinstance(time_signature_1, TimeSignature)
    assert isinstance(time_signature_2, TimeSignature)
    assert format(time_signature_1, 'lilypond') == '\\time 9/32'
    assert format(time_signature_2, 'lilypond') == '\\time 9/32'
    assert time_signature_1 is not time_signature_2


def test_indicatortools_TimeSignature___init___03():
    r'''Initializes time signature from other time signature
    instance with partial.
    '''

    time_signature_1 = TimeSignature((9, 32), partial=Duration(1, 32))
    time_signature_2 = TimeSignature(time_signature_1)

    assert time_signature_1 == time_signature_2
    assert not time_signature_1 is time_signature_2


# -*- encoding: utf-8 -*-
from abjad.tools import abctools


class MeterManager(abctools.AbjadObject):
    r'''A meter manager.
    '''

    ### PUBLIC METHODS ###

    @staticmethod
    def is_acceptable_logical_tie(
        logical_tie_duration=None,
        logical_tie_starts_in_offsets=None,
        logical_tie_stops_in_offsets=None,
        maximum_dot_count=None,
        ):
        r'''Is true if logical tie is acceptable.
        '''
        #print '\tTESTING ACCEPTABILITY'
        if not logical_tie_duration.is_assignable:
            return False
        if maximum_dot_count is not None and \
            maximum_dot_count < logical_tie_duration.dot_count:
            return False
        if not logical_tie_starts_in_offsets and \
            not logical_tie_stops_in_offsets:
            return False
        return True

    @staticmethod
    def is_boundary_crossing_logical_tie(
        boundary_depth=None,
        boundary_offsets=None,
        logical_tie_start_offset=None,
        logical_tie_stop_offset=None,
        ):
        r'''Is true if logical tie crosses meter boundaries.
        '''
        #print '\tTESTING BOUNDARY CROSSINGS'
        if boundary_depth is None:
            return False
        if not any(logical_tie_start_offset < x < logical_tie_stop_offset
            for x in boundary_offsets):
            return False
        if logical_tie_start_offset in boundary_offsets and \
            logical_tie_stop_offset in boundary_offsets:
            return False
        return True

    @staticmethod
    def iterate_rewrite_inputs(expr):
        r'''Iterate topmost masked logical ties, rest groups and containers in
        `expr`, masked by `expr`:

        ::

            >>> string = "abj: | 2/4 c'4 d'4 ~ |"
            >>> string += "| 4/4 d'8. r16 r8. e'16 ~ "
            >>> string += "2/3 { e'8 ~ e'8 f'8 ~ } f'4 ~ |"
            >>> string += "| 4/4 f'8 g'8 ~ g'4 a'4 ~ a'8 b'8 ~ |"
            >>> string += "| 2/4 b'4 c''4 |"
            >>> staff = Staff(string)

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                {
                    \time 2/4
                    c'4
                    d'4 ~
                }
                {
                    \time 4/4
                    d'8.
                    r16
                    r8.
                    e'16 ~
                    \times 2/3 {
                        e'8 ~
                        e'8
                        f'8 ~
                    }
                    f'4 ~
                }
                {
                    f'8
                    g'8 ~
                    g'4
                    a'4 ~
                    a'8
                    b'8 ~
                }
                {
                    \time 2/4
                    b'4
                    c''4
                }
            }

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[0]): x
            ...
            LogicalTie(Note("c'4"),)
            LogicalTie(Note("d'4"),)

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[1]): x
            ...
            LogicalTie(Note("d'8."),)
            LogicalTie(Rest('r16'), Rest('r8.'))
            LogicalTie(Note("e'16"),)
            Tuplet(Multiplier(2, 3), "e'8 ~ e'8 f'8 ~")
            LogicalTie(Note("f'4"),)

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[2]): x
            ...
            LogicalTie(Note("f'8"),)
            LogicalTie(Note("g'8"), Note("g'4"))
            LogicalTie(Note("a'4"), Note("a'8"))
            LogicalTie(Note("b'8"),)

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[3]): x
            ...
            LogicalTie(Note("b'4"),)
            LogicalTie(Note("c''4"),)

        Returns generator.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import spannertools

        last_tie_spanner = None
        current_leaf_group = None
        current_leaf_group_is_silent = False

        for x in expr:
            if isinstance(x, (scoretools.Note, scoretools.Chord)):
                this_tie_spanner = x._get_spanners(spannertools.Tie) or None
                if current_leaf_group is None:
                    current_leaf_group = []
                elif current_leaf_group_is_silent or \
                    this_tie_spanner is None or \
                    last_tie_spanner != this_tie_spanner:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = []
                current_leaf_group_is_silent = False
                current_leaf_group.append(x)
                last_tie_spanner = this_tie_spanner
            elif isinstance(x, (scoretools.Rest, scoretools.Skip)):
                if current_leaf_group is None:
                    current_leaf_group = []
                elif not current_leaf_group_is_silent:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = []
                current_leaf_group_is_silent = True
                current_leaf_group.append(x)
                last_tie_spanner = None
            elif isinstance(x, scoretools.Container):
                if current_leaf_group is not None:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = None
                    last_tie_spanner = None
                yield x

            else:
                message = 'unhandled component: {!r}.'
                message = message.format(x)
                raise Exception(message)
        if current_leaf_group is not None:
            yield selectiontools.LogicalTie(current_leaf_group)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class AttributeManifest(AbjadObject):
    r'''Target manifest.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_attribute_details',
        )

    ### INITIALIZER ###

    def __init__(self, *attribute_details):
        from abjad.tools import systemtools
        self._attribute_details = []
        for attribute_detail in attribute_details:
            if not isinstance(attribute_detail, systemtools.AttributeDetail):
                attribute_detail = systemtools.AttributeDetail(
                    *attribute_detail)
            self.attribute_details.append(attribute_detail)

    ### SPECIAL METHODS ###

    def __getitem__(self, expr):
        r'''Gets attribute detail from attribute manifest.

        Returns attribute detail.
        '''
        return self.attribute_details.__getitem__(expr)

    def __repr__(self):
        r'''Gets interpreter representation of target manifest.

        Returns string.
        '''
        parts = ', '.join([str(x) for x in self.attribute_details])
        result = '{}({})'
        result = result.format(type(self).__name__, parts)
        return result

    ### PRIVATE METHODS ###

    def _command_to_attribute_detail(self, command):
        for attribute_detail in self.attribute_details:
            if attribute_detail.command == command:
                return attribute_detail

    def _command_to_attribute_name(self, command):
        attribute_detail = self._command_to_attribute_detail(command)
        if attribute_detail:
            return attribute_detail.name

    def _command_to_prepopulated_value(self, command):
        attribute_name = self._command_to_attribute_name(command)
        return getattr(self.target, attribute_name, None)

    def _to_initializer_argument_names(self, retrievable_attribute_name):
        for attribute_detail in self.attribute_details:
            if attribute_detail.name == retrievable_attribute_name:
                return attribute_detail.name
        raise ValueError

    def _to_retrievable_attribute_name(self, initializer_argument_name):
        for attribute_detail in self.attribute_details:
            if attribute_detail.name == initializer_argument_name:
                return attribute_detail.name
        raise ValueError

    ### PUBLIC PROPERTIES ###

    @property
    def attribute_details(self):
        r'''Gets attribute details of target manifest.

        Returns list.
        '''
        return self._attribute_details


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_make_centered_title_markup_01():

    markup = markuptools.make_centered_title_markup('String Quartet')

    assert systemtools.TestManager.compare(
        format(markup, 'lilypond'),
        r'''
        \markup {
            \override
                #'(font-name . "Times")
                \fontsize
                    #18
                    \column
                        {
                            \center-align
                                {
                                    {
                                        \vspace
                                            #6
                                        \line
                                            {
                                                "String Quartet"
                                            }
                                        \vspace
                                            #12
                                    }
                                }
                        }
            }
        ''',
        )


def test_markuptools_make_centered_title_markup_02():
    r'''List of multiple title lines.
    '''

    markup = markuptools.make_centered_title_markup(
        ['String Quartet', 'for the JACK Quartet'],
        )

    assert systemtools.TestManager.compare(
        format(markup, 'lilypond'),
        r'''
        \markup {
            \override
                #'(font-name . "Times")
                \fontsize
                    #18
                    \column
                        {
                            \center-align
                                {
                                    {
                                        \vspace
                                            #6
                                        \line
                                            {
                                                "String Quartet"
                                            }
                                        \line
                                            {
                                                "for the JACK Quartet"
                                            }
                                        \vspace
                                            #12
                                    }
                                }
                        }
            }
        ''',
        )


# -*- encoding: utf-8 -*-
import copy
from abjad.tools import mathtools


def partition_sequence_by_counts(
    sequence,
    counts,
    cyclic=False,
    overhang=False,
    copy_elements=False,
    ):
    r'''Partitions sequence by counts.

    ..  container:: example

        **Example 1a.** Partition sequence once by counts without overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(10)),
            ...     [3],
            ...     cyclic=False,
            ...     overhang=False,
            ...     )
            [[0, 1, 2]]

    ..  container:: example

        **Example 1b.** Partition sequence once by counts without overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(16)),
            ...     [4, 3],
            ...     cyclic=False,
            ...     overhang=False,
            ...     )
            [[0, 1, 2, 3], [4, 5, 6]]

    ..  container:: example

        **Example 2a.** Partition sequence cyclically by counts without
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(10)),
            ...     [3],
            ...     cyclic=True,
            ...     overhang=False,
            ...     )
            [[0, 1, 2], [3, 4, 5], [6, 7, 8]]

    ..  container:: example

        **Example 2b.** Partition sequence cyclically by counts without
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(16)),
            ...     [4, 3],
            ...     cyclic=True,
            ...     overhang=False,
            ...     )
            [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10], [11, 12, 13]]

    ..  container:: example

        **Example 3a.** Partition sequence once by counts with overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(10)),
            ...     [3],
            ...     cyclic=False,
            ...     overhang=True,
            ...     )
            [[0, 1, 2], [3, 4, 5, 6, 7, 8, 9]]

    ..  container:: example

        **Example 3b.** Partition sequence once by counts with overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(16)),
            ...     [4, 3],
            ...     cyclic=False,
            ...     overhang=True,
            ...     )
            [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10, 11, 12, 13, 14, 15]]

    ..  container:: example

        **Example 4a.** Partition sequence cyclically by counts with overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(10)),
            ...     [3],
            ...     cyclic=True,
            ...     overhang=True,
            ...     )
            [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]

    ..  container:: example

        **Example 4b.** Partition sequence cyclically by counts with overhang:

        ::

            >>> sequencetools.partition_sequence_by_counts(
            ...     list(range(16)),
            ...     [4, 3],
            ...     cyclic=True,
            ...     overhang=True,
            ...     )
            [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10], [11, 12, 13], [14, 15]]

    Returns list of sequence objects.
    '''
    from abjad.tools import sequencetools

    result = []

    if cyclic:
        if overhang:
            counts = sequencetools.repeat_sequence_to_weight(
                counts, len(sequence))
        else:
            counts = sequencetools.repeat_sequence_to_weight(
                counts, len(sequence), allow_total=Less)
    elif overhang:
        weight_counts = mathtools.weight(counts)
        len_sequence = len(sequence)
        if weight_counts < len_sequence:
            counts = list(counts)
            counts.append(len(sequence) - weight_counts)

    for start, stop in mathtools.cumulative_sums_pairwise(counts):
        result.append(type(sequence)(sequence[start:stop]))

    return result



# -*- encoding: utf-8 -*-
import argparse
import os
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
from abjad.tools.developerscripttools.CleanScript import CleanScript


class AbjGrepScript(DirectoryScript):
    r'''Runs `grep` against a path, ignoring `svn` and docs-related files.

    ..  shell::

        ajv grep --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'grep'``.
        '''
        return 'grep'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string.
        '''
        return '''\
If no PATH flag is specified, the current directory will be searched.
    '''

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'grep PATTERN in PATH'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''

        systemtools.IOManager.clear_terminal()
        if args.whole_words_only:
            whole_words_only = '-w'
        else:
            whole_words_only = ''
        command = r'grep {} -Irn {!r} {} | grep -v svn-base | grep -v svn\/ | grep -v docs'.format(
            whole_words_only, args.pattern, os.path.relpath(args.path))
        systemtools.IOManager.spawn_subprocess(command)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''

        from abjad import abjad_configuration

        parser.add_argument('pattern',
            help='pattern to search for'
            )

        parser.add_argument('-W', '--whole-words-only',
            action='store_true',
            help='''match only whole words, similar to grep's "-w" flag''',
            )

        group = parser.add_mutually_exclusive_group()

        group.add_argument('-P', '--path',
            dest='path',
            help='grep PATH',
            type=self._validate_path,
            )

        group.add_argument('-X', '--experimental',
            action='store_const',
            const=abjad_configuration.abjad_experimental_directory,
            dest='path',
            help='grep Abjad abjad.tools directory',
            )

        group.add_argument('-M', '--mainline',
            action='store_const',
            const=abjad_configuration.abjad_directory,
            dest='path',
            help='grep Abjad mainline directory',
            )

        group.add_argument('-T', '--tools',
            action='store_const',
            const=os.path.join(
                abjad_configuration.abjad_directory, 'tools'),
            dest='path',
            help='grep Abjad mainline tools directory',
            )

        group.add_argument('-R', '--root',
            action='store_const',
            const=abjad_configuration.abjad_root_directory,
            dest='path',
            help='grep Abjad root directory',
            )

        parser.set_defaults(path=os.path.abspath(os.path.curdir))


# -*- encoding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_components_and_grace_containers_01():
    r'''Yield before-grace notes and after-grace notes.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16")]
    grace = scoretools.GraceContainer(notes[:2], kind='grace')
    attach(grace, voice[1])
    after_grace = scoretools.GraceContainer(notes[2:], kind='after')
    attach(after_grace, voice[1])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            \grace {
                c'16
                d'16
            }
            \afterGrace
            d'8
            {
                e'16
                f'16
            }
            e'8
            f'8 ]
        }
        ''',
        )

    notes = iterate(voice).by_components_and_grace_containers(Note)
    notes = list(notes)

    assert notes[0] is voice[0]
    assert notes[1] is grace[0]
    assert notes[2] is grace[1]
    assert notes[3] is voice[1]
    assert notes[4] is after_grace[0]
    assert notes[5] is after_grace[1]
    assert notes[6] is voice[2]
    assert notes[7] is voice[3]


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_pitch_class_numbers_01():
    r'''With number=True.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_pitch_class_numbers(
        staff, number=True)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
                _ \markup {
                    \small
                        0
                    }
            d'8
                _ \markup {
                    \small
                        2
                    }
            e'8
                _ \markup {
                    \small
                        4
                    }
            f'8
                _ \markup {
                    \small
                        5
                    }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_labeltools_label_leaves_in_expr_with_pitch_class_numbers_02():
    r'''With color=True.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_pitch_class_numbers(
        staff, number=False, color=True)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \once \override NoteHead #'color = #(x11-color 'red)
            c'8
            \once \override NoteHead #'color = #(x11-color 'orange)
            d'8
            \once \override NoteHead #'color = #(x11-color 'ForestGreen)
            e'8
            \once \override NoteHead #'color = #(x11-color 'MediumOrchid)
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchArray_to_measures_01():

    array = pitchtools.PitchArray([
        [1, (2, 1), ([-2, -1.5], 2)],
        [(7, 2), (6, 1), 1],
        ])

    '''
    [  ] [d'] [bf bqf     ]
    [g'      ] [fs'    ] []
    '''

    measures = array.to_measures()
    score = Score(Staff([]) * 2)
    score[0].append(measures[0])
    score[1].append(measures[1])

    r'''
    \new Score <<
        \new Staff {
            {
                \time 4/8
                r8
                d'8
                <bf bqf>4
            }
        }
        \new Staff {
            {
                \time 4/8
                g'4
                fs'8
                r8
            }
        }
    >>
    '''

    assert inspect_(score).is_well_formed()
    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new Staff {
                {
                    \time 4/8
                    r8
                    d'8
                    <bf bqf>4
                }
            }
            \new Staff {
                {
                    \time 4/8
                    g'4
                    fs'8
                    r8
                }
            }
        >>
        '''
        )


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import new


class RestRhythmMaker(RhythmMaker):
    r'''Rest rhythm-maker.

    ..  container:: example

        Makes rests equal to the duration of input divisions.

        ::

            >>> maker = rhythmmakertools.RestRhythmMaker()

        ::

            >>> divisions = [(5, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 5/16
                    r4
                    r16
                }
                {
                    \time 3/8
                    r4.
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _class_name_abbreviation = 'R'

    _human_readable_class_name = 'rest rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        duration_spelling_specifier=None,
        ):
        RhythmMaker.__init__(
            self,
            duration_spelling_specifier=duration_spelling_specifier,
            )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls rest rhythm-maker on `divisions`.

        Returns list of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __format__(self, format_specification=''):
        r'''Formats rest rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> print(format(maker))
                rhythmmakertools.RestRhythmMaker()

        Returns string.
        '''
        superclass = super(RestRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import rhythmmakertools
        from abjad.tools import systemtools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='duration_spelling_specifier',
                command='dss',
                editor=rhythmmakertools.DurationSpellingSpecifier,
                ),
            )

    ### PRIVATE METHODS ###

    def _make_music(self, duration_pairs, seeds):
        from abjad.tools import rhythmmakertools
        result = []
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        for duration_pair in duration_pairs:
            rests = scoretools.make_leaves(
                pitches=None,
                durations=[duration_pair],
                decrease_durations_monotonically=\
                    specifier.decrease_durations_monotonically,
                forbidden_written_duration=\
                    specifier.forbidden_written_duration,
                )
            result.append(rests)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier of rest rhythm-maker.

            ..  container:: example

                Forbids rests with written duration greater than or equal to
                ``1/4`` of a whole note:

                ::

                    >>> duration_spelling_specifier = \
                    ...     rhythmmakertools.DurationSpellingSpecifier(
                    ...     forbidden_written_duration=Duration(1, 4),
                    ...     )
                    >>> maker = rhythmmakertools.RestRhythmMaker(
                    ...     duration_spelling_specifier=duration_spelling_specifier,
                    ...     )

                ::

                    >>> divisions = [(5, 16), (3, 8)]
                    >>> music = maker(divisions)
                    >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                    ...     music,
                    ...     divisions,
                    ...     )
                    >>> show(lilypond_file) # doctest: +SKIP

                ..  doctest::

                    >>> staff = maker._get_rhythmic_staff(lilypond_file)
                    >>> f(staff)
                    \new RhythmicStaff {
                        {
                            \time 5/16
                            r8
                            r8
                            r16
                        }
                        {
                            \time 3/8
                            r8
                            r8
                            r8
                        }
                    }

        Returns duration spelling specifier or none.
        '''
        return RhythmMaker.duration_spelling_specifier.fget(self)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses rest rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.RestRhythmMaker()
                >>> reversed_maker = maker.reverse()

            ::

                >>> print(format(reversed_maker))
                rhythmmakertools.RestRhythmMaker(
                    duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                        decrease_durations_monotonically=False,
                        ),
                    )

            ::

                >>> divisions = [(5, 16), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/16
                        r4
                        r16
                    }
                    {
                        \time 3/8
                        r4.
                    }
                }

        Returns new rest rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            default = rhythmmakertools.DurationSpellingSpecifier()
            duration_spelling_specifier = default
        duration_spelling_specifier = duration_spelling_specifier.reverse()
        arguments = {
            'duration_spelling_specifier': duration_spelling_specifier,
            }
        result = new(self, **arguments)
        return result


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class Matrix(AbjadObject):
    '''A matrix.

    Initializes from rows:

    ::

        >>> matrix = datastructuretools.Matrix([
        ...     [0, 1, 2, 3],
        ...     [10, 11, 12, 13],
        ...     [20, 21, 22, 23],
        ...     ])

    ::

        >>> matrix
        Matrix(3x4)

    ::

        >>> matrix[:]
        ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))

    ::

        >>> matrix[2]
        (20, 21, 22, 23)

    ::

        >>> matrix[2][0]
        20

    Initializes from columns:

    ::

        >>> matrix = datastructuretools.Matrix(columns=[
        ...     [0, 10, 20],
        ...     [1, 11, 21],
        ...     [2, 12, 22],
        ...     [3, 13, 23],
        ...     ])

    ::

        >>> matrix
        Matrix(3x4)

    ::

        >>> matrix[:]
        ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))

    ::

        >>> matrix[2]
        (20, 21, 22, 23)

    ::

        >>> matrix[2][0]
        20

    Matrix currently implements only item retrieval.

    Concatenation and division remain to be implemented.

    Standard transforms of linear algebra remain to be implemented.
    '''

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        if len(args) == 1:
            assert not kwargs
            rows, columns = self._initialize_from_rows(args[0])
        elif 'columns' in kwargs:
            assert not args
            rows, columns = self._initialize_from_columns(kwargs['columns'])
        elif len(args) == 0:
            rows, columns = self._initialize_from_rows([[0], [0]])
        else:
            message = 'can not initialize {}: {!r}.'
            message = message(type(self).__name__, args)
            raise ValueError(message)
        self._rows = rows
        self._columns = columns
        self._n_rows = len(rows)
        self._n_columns = len(columns)

    ### SPECIAL METHODS ###

    def __getitem__(self, i):
        r'''Gets row `i` from matrix.

        ::

            >>> matrix[1]
            (10, 11, 12, 13)

        Returns row.
        '''
        return self.rows[i]

    def __repr__(self):
        r'''Gets interpreter representation of matrix.

        ::

            >>> matrix
            Matrix(3x4)

        Returns string.
        '''
        return '{}({}x{})'.format(
            type(self).__name__,
            self._n_rows,
            self._n_columns,
            )

    ### PRIVATE METHODS ###

    def _initialize_from_columns(self, columns):
        columns = tuple([tuple(column) for column in columns])
        assert len(set([len(column) for column in columns])) in (0, 1)
        rows = []
        for row_index in range(len(columns[0])):
            row = tuple([column[row_index] for column in columns])
            rows.append(row)
        rows = tuple(rows)
        return rows, columns

    def _initialize_from_rows(self, rows):
        rows = tuple([tuple(row) for row in rows])
        assert len(set([len(row) for row in rows])) in (0, 1)
        columns = []
        for column_index in range(len(rows[0])):
            column = tuple([row[column_index] for row in rows])
            columns.append(column)
        columns = tuple(columns)
        return rows, columns

    ### PUBLIC PROPERTIES ###

    @property
    def columns(self):
        r'''Columns of matrix.

        ::

            >>> matrix = datastructuretools.Matrix(
            ...     [[0, 1, 2, 3],
            ...     [10, 11, 12, 13],
            ...     [20, 21, 22, 23],
            ...     ])

        ::

            >>> matrix.columns
            ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))

        Returns tuple.
        '''
        return self._columns

    @property
    def rows(self):
        r'''Rows of matrix.

        ::

            >>> matrix = datastructuretools.Matrix(
            ...     [[0, 1, 2, 3],
            ...     [10, 11, 12, 13],
            ...     [20, 21, 22, 23],
            ...     ])

        ::

            >>> matrix.rows
            ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))

        Returns tuple.
        '''
        return self._rows


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject
from abjad.tools import durationtools
from abjad.tools.topleveltools import new


class SpacingIndication(AbjadObject):
    r'''Spacing indication token.

    LilyPond ``Score.proportionalNotationDuration``
    will equal ``proportional_notation_duration`` when tempo
    equals ``tempo_indication``.

    Initialize from tempo and proportional notation duration:

    ::

        >>> tempo = Tempo(Duration(1, 8), 44)
        >>> indication = layouttools.SpacingIndication(tempo, Duration(1, 68))

    ::

        >>> indication
        SpacingIndication(Tempo(duration=Duration(1, 8), units_per_minute=44), Duration(1, 68))

    Initialize from constants:

    ::

        >>> layouttools.SpacingIndication(((1, 8), 44), (1, 68))
        SpacingIndication(Tempo(duration=Duration(1, 8), units_per_minute=44), Duration(1, 68))

    Initialize from other spacing indication:

    ::

        >>> layouttools.SpacingIndication(indication)
        SpacingIndication(Tempo(duration=Duration(1, 8), units_per_minute=44), Duration(1, 68))

    Spacing indications are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_proportional_notation_duration',
        '_tempo_indication',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import indicatortools
        if len(args) == 1 and isinstance(args[0], type(self)):
            self._tempo_indication = args[0].tempo_indication
            self._proportional_notation_duration = \
                args[0].proportional_notation_duration
        elif len(args) == 2:
            tempo = args[0]
            if isinstance(tempo, tuple):
                tempo = indicatortools.Tempo(*tempo)
            tempo_indication = tempo
            proportional_notation_duration = durationtools.Duration(args[1])
            self._tempo_indication = tempo_indication
            self._proportional_notation_duration = \
                proportional_notation_duration
        elif len(args) == 0:
            tempo = indicatortools.Tempo()
            proportional_notation_duration = durationtools.Duration(1, 68)
            self._tempo_indication = tempo
            self._proportional_notation_duration = \
                proportional_notation_duration
        else:
            message = 'can not initialize spacing indication from {!r}'
            message = message.format(args)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Spacing indications compare equal when normalized
        spacing durations compare equal.
        '''
        if isinstance(expr, SpacingIndication):
            if self.normalized_spacing_duration == \
                expr.normalized_spacing_duration:
                return True
        return False

    def __hash__(self):
        r'''Hashes spacing indication.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(SpacingIndication, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return new(
            self._storage_format_specification,
            is_indented=False,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                self._tempo_indication,
                self._proportional_notation_duration,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def normalized_spacing_duration(self):
        r'''Proportional notation duration normalized to 60 MM.

        Returns duration.
        '''
        indication = self.tempo_indication
        duration = self.proportional_notation_duration
        scalar = indication.duration / indication.units_per_minute * \
            60 / durationtools.Duration(1, 4)
        return scalar * self.proportional_notation_duration

    @property
    def proportional_notation_duration(self):
        r'''LilyPond proportional notation duration of spacing indication.

        Returns duration.
        '''
        return self._proportional_notation_duration

    @property
    def tempo_indication(self):
        r'''Tempo of spacing indication.

        Returns tempo.
        '''
        return self._tempo_indication


# -*- encoding: utf-8 -*-
'''Abjad ``pitchtools`` package.

    Dependencies:

    The ``pitchtools`` package may import ``sequencetools`` at top level.

    The ``pitchtools`` package should not import any component packages
    at top level.

    The ``pitchtools`` package should not import ``indicatortools`` at
    top level, and vice versa.

'''
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange_one_line_numbered_pitch_repr_01():

    pitch_range = pitchtools.PitchRange.from_pitches(-12, 36)
    assert pitch_range.one_line_numbered_pitch_repr == '[-12, 36]'


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_nonnegative_integer_powers_of_two_01():
    r'''Is true when all elements in sequence are nonnegative integer
    powers of two.
    '''

    numbers = [1, 2, 256, 8, 16, 16, 16]
    assert mathtools.all_are_nonnegative_integer_powers_of_two(numbers)


def test_mathtools_all_are_nonnegative_integer_powers_of_two_02():
    r'''True on empty sequence.
    '''

    assert mathtools.all_are_nonnegative_integer_powers_of_two([])


def test_mathtools_all_are_nonnegative_integer_powers_of_two_03():
    r'''Otherwise false.
    '''

    assert not mathtools.all_are_nonnegative_integer_powers_of_two([3])

    numbers = [1, 2, 4, 8, 16, 17]
    assert not mathtools.all_are_nonnegative_integer_powers_of_two(numbers)


def test_mathtools_all_are_nonnegative_integer_powers_of_two_04():
    r'''False when expr is not a sequence.
    '''

    assert not mathtools.all_are_nonnegative_integer_powers_of_two(16)


# -*- encoding: utf-8 -*-
from abjad import *


def test_selectiontools_SliceSelection_group_by_01():

    staff = Staff(scoretools.make_leaves([0, 2, 4, None, None, 5, 7], [(1, 8)]))
    t = staff.select_leaves().group_by(type)

    assert t == [(staff[0], staff[1], staff[2]), (staff[3], staff[4]),
        (staff[5], staff[6])]


# -*- encoding: utf-8 -*-
import sys


def replace_elements(
    sequence,
    indices,
    new_material,
    ):
    '''Replaces `sequence` elements.

    ..  container:: example

        Replaces elements at indices 0, 2, 4, 6 with ``'A'``, ``'B'``,
        ``'C'``, ``'D'``, respectively:

        ::

            >>> sequencetools.replace_elements(
            ...     list(range(16)),
            ...     ([0], 2),
            ...     (['A', 'B', 'C', 'D'], None),
            ...     )
            ['A', 1, 'B', 3, 'C', 5, 'D', 7, 8, 9, 10, 11, 12, 13, 14, 15]

    ..  container:: example

        Replaces elements at indices 0, 1, 8, 13 with ``'A'``, ``'B'``,
        ``'C'``, ``'D'``, respectively:

        ::

            >>> sequencetools.replace_elements(
            ...     list(range(16)),
            ...     ([0, 1, 8, 13], None),
            ...     (['A', 'B', 'C', 'D'], None),
            ...     )
            ['A', 'B', 2, 3, 4, 5, 6, 7, 'C', 9, 10, 11, 12, 'D', 14, 15]

    ..  container:: example

        Replaces every element at an even index with ``'*'``:

        ::

            >>> sequencetools.replace_elements(
            ...     list(range(16)),
            ...     ([0], 2),
            ...     (['*'], 1),
            ...     )
            ['*', 1, '*', 3, '*', 5, '*', 7, '*', 9, '*', 11, '*', 13, '*', 15]

    ..  container:: example

        Replaces every element at an index congruent to 0 (mod 6) with ``'A'``;
        replaces every element at an index congruent to 2 (mod 6) with ``'B'``:

        ::

            >>> sequencetools.replace_elements(
            ...     list(range(16)),
            ...     ([0], 2),
            ...     (['A', 'B'], 3),
            ...     )
            ['A', 1, 'B', 3, 4, 5, 'A', 7, 'B', 9, 10, 11, 'A', 13, 'B', 15]

    Raises type error when `sequence` is not a list.

    Returns object of `sequence` type.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    assert isinstance(indices, tuple) and len(indices) == 2
    index_values, index_period = indices

    assert isinstance(index_values, list)
    assert isinstance(index_period, (int, type(None)))

    assert isinstance(new_material, tuple) and len(new_material) == 2
    material_values, material_period = new_material

    assert isinstance(material_values, list)
    assert isinstance(material_period, (int, type(None)))

    try:
        maxint = sys.maxint
    except AttributeError:
        maxint = sys.maxsize

    if index_period is None:
        index_period = maxint

    if material_period is None:
        material_period = maxint

    result = []

    material_index = 0

    for index, element in enumerate(sequence):
        if index % index_period in index_values:
            try:
                cyclic_material_index = material_index % material_period
                material_value = material_values[cyclic_material_index]
                result.append(material_value)
            except IndexError:
                result.append(element)
            material_index += 1
        else:
            result.append(element)

    result = type(sequence)(result)
    return result



# -*- encoding: utf-8 -*-
import fractions
import numbers


def divide_number_by_ratio(number, ratio):
    r'''Divides integer by `ratio`.

    ::

        >>> mathtools.divide_number_by_ratio(1, [1, 1, 3])
        [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

    Divides fraction by `ratio`:

    ::

        >>> mathtools.divide_number_by_ratio(Fraction(1), [1, 1, 3])
        [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

    Divides float by ratio:

    ::

        >>> mathtools.divide_number_by_ratio(1.0, [1, 1, 3]) # doctest: +SKIP
        [0.20000000000000001, 0.20000000000000001, 0.60000000000000009]

    Raises type error on nonnumeric `number`.

    Raises type error on noninteger in `ratio`.

    Returns list of fractions or list of floats.
    '''
    from abjad.tools import mathtools

    # check input
    assert isinstance(number, numbers.Number)
    ratio = mathtools.Ratio(ratio)

    # find factors and multiply by factors
    factors = [fractions.Fraction(p, sum(ratio)) for p in ratio]
    result = [factor * number for factor in factors]

    # return result
    return result


# -*- encoding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie_all_leaves_are_in_same_parent_01():


    staff = scoretools.make_repeated_notes(4)
    tie = spannertools.Tie()
    attach(tie, staff[:])

    assert inspect_(staff[0]).get_logical_tie().all_leaves_are_in_same_parent


def test_selectiontools_LogicalTie_all_leaves_are_in_same_parent_02():

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    tie = spannertools.Tie()
    attach(tie, staff.select_leaves()[1:3])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                c'8 ~
            }
            {
                c'8
                c'8
            }
        }
        '''
        )

    assert inspect_(staff.select_leaves()[0]).get_logical_tie().all_leaves_are_in_same_parent
    assert not inspect_(staff.select_leaves()[1]).get_logical_tie().all_leaves_are_in_same_parent
    assert not inspect_(staff.select_leaves()[2]).get_logical_tie().all_leaves_are_in_same_parent
    assert inspect_(staff.select_leaves()[3]).get_logical_tie().all_leaves_are_in_same_parent


def test_selectiontools_LogicalTie_all_leaves_are_in_same_parent_03():

    staff = Staff(r"\times 2/3 { c'8 c'8 c'8 ~ } \times 2/3 { c'8 c'8 c'8 }")

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \times 2/3 {
                c'8
                c'8
                c'8 ~
            }
            \times 2/3 {
                c'8
                c'8
                c'8
            }
        }
        '''
        )

    assert inspect_(staff.select_leaves()[0]).get_logical_tie().all_leaves_are_in_same_parent
    assert inspect_(staff.select_leaves()[1]).get_logical_tie().all_leaves_are_in_same_parent
    assert not inspect_(staff.select_leaves()[2]).get_logical_tie().all_leaves_are_in_same_parent
    assert not inspect_(staff.select_leaves()[3]).get_logical_tie().all_leaves_are_in_same_parent
    assert inspect_(staff.select_leaves()[4]).get_logical_tie().all_leaves_are_in_same_parent
    assert inspect_(staff.select_leaves()[5]).get_logical_tie().all_leaves_are_in_same_parent


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools


def parse_rtm_syntax(rtm):
    r'''Parses RTM syntax.

    ::

        >>> rtm = '(1 (1 (1 (1 1)) 1))'
        >>> rhythmtreetools.parse_rtm_syntax(rtm)
        FixedDurationTuplet(Duration(1, 4), "c'8 c'16 c'16 c'8")

    Also supports fractional durations:

    ::

        >>> rtm = '(3/4 (1 1/2 (4/3 (1 -1/2 1))))'
        >>> rhythmtreetools.parse_rtm_syntax(rtm)
        FixedDurationTuplet(Duration(3, 16), 'c\'8 c\'16 FixedDurationTuplet(Duration(1, 6), "c\'8 r16 c\'8")')

    ::

        >>> print(format(_))
        \tweak #'text #tuplet-number::calc-fraction-text
        \times 9/17 {
            c'8
            c'16
            \tweak #'edge-height #'(0.7 . 0)
            \times 8/15 {
                c'8
                r16
                c'8
            }
        }

    Returns fixed-duration tuplet or container.
    '''
    from abjad.tools import rhythmtreetools

    result = rhythmtreetools.RhythmTreeParser()(rtm)

    con = scoretools.Container()

    for node in result:
        tuplet = node((1, 4))
        # following line added 2012-08-01. tb.
        tuplet = tuplet[0]
        if tuplet.is_trivial:
            con.extend(tuplet[:])
        else:
            con.append(tuplet)

    if len(con) == 1:
        return con[0]
    return con



# -*- encoding: utf-8 -*-
import sys


def partition_integer_by_ratio(n, ratio):
    r'''Partitions positive integer-equivalent `n` by `ratio`.

    ::

        >>> mathtools.partition_integer_by_ratio(10, [1, 2])
        [3, 7]

    Partitions positive integer-equivalent `n` by `ratio` with negative parts:

    ::

        >>> mathtools.partition_integer_by_ratio(10, [1, -2])
        [3, -7]

    Partitions negative integer-equivalent `n` by `ratio`:

    ::

        >>> mathtools.partition_integer_by_ratio(-10, [1, 2])
        [-3, -7]

    Partitions negative integer-equivalent `n` by `ratio` with negative parts:

    ::

        >>> mathtools.partition_integer_by_ratio(-10, [1, -2])
        [-3, 7]

    Returns result with weight equal to absolute value of `n`.

    Raises type error on noninteger `n`.

    Returns list of integers.
    '''
    from abjad.tools import mathtools

    if not mathtools.is_integer_equivalent_number(n):
        message = 'is not integer-equivalent number: {!r}.'
        message = message.format(n)
        raise TypeError(message)

    if not all(mathtools.is_integer_equivalent_number(part) for part in ratio):
        message = 'some parts in {!r} not integer-equivalent numbers.'
        message = message.format(ratio)
        raise TypeError(message)

    result = [0]

    divisions = [
        float(abs(n)) * abs(part) / mathtools.weight(ratio)
        for part in ratio
        ]
    cumulative_divisions = mathtools.cumulative_sums(divisions, start=None)

    for division in cumulative_divisions:
        rounded_division = int(round(division)) - sum(result)
        #This makes rounding behave like python 2. Would be good to remove
        # in the long run
        if sys.version_info[0] == 3:
            if division - round(division) == 0.5:
                rounded_division += 1
        result.append(rounded_division)

    result = result[1:]

    # adjust signs of output elements
    if mathtools.sign(n) == -1:
        result = [-x for x in result]
    ratio_signs = [mathtools.sign(x) for x in ratio]
    result = [pair[0] * pair[1] for pair in zip(ratio_signs, result)]



    # return result
    return result



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools.rhythmtreetools.RhythmTreeNode import RhythmTreeNode


class RhythmTreeLeaf(RhythmTreeNode):
    r'''A rhythm-tree leaf.

    ::

        >>> leaf = rhythmtreetools.RhythmTreeLeaf(
        ...     preprolated_duration=5, is_pitched=True)
        >>> leaf
        RhythmTreeLeaf(
            preprolated_duration=Duration(5, 1),
            is_pitched=True
            )

    Call with a pulse preprolated_duration to generate Abjad leaf objects:

    ::

        >>> result = leaf((1, 8))
        >>> result
        Selection(Note("c'2"), Note("c'8"))

    Generates rests when called, if `is_pitched` is False:

    ::

        >>> rhythmtreetools.RhythmTreeLeaf(
        ...     preprolated_duration=7, is_pitched=False)((1, 16))
        Selection(Rest('r4..'),)

    '''

    ### INITIALIZER ###

    def __init__(
        self,
        preprolated_duration=1,
        is_pitched=True,
        name=None,
        ):
        RhythmTreeNode.__init__(
            self,
            preprolated_duration=preprolated_duration,
            name=name,
            )
        self.is_pitched = is_pitched

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        r'''Generate Abjad score components:

        ::

            >>> leaf = rhythmtreetools.RhythmTreeLeaf(5)
            >>> leaf((1, 4))
            Selection(Note("c'1"), Note("c'4"))

        Returns sequence of components.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.preprolated_duration
        if self.is_pitched:
            return scoretools.make_notes(0, total_duration)
        return scoretools.make_rests(total_duration)

    def __eq__(self, expr):
        r'''Is true when `expr` is a rhythm tree leaf with preprolated duration
        and pitch boolean equal to those of this rhythm tree leaf. Otherwise
        false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.preprolated_duration == expr.preprolated_duration:
                if self.is_pitched == expr.is_pitched:
                    return True
        return False

    def __hash__(self):
        r'''Hashes rhythm-tree leaf.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(RhythmTreeLeaf, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.preprolated_duration)]

    ### PUBLIC PROPERTIES ###

    @property
    def graphviz_graph(self):
        r'''Graphviz graph of rhythm tree leaf.
        '''
        graph = documentationtools.GraphvizGraph(name='G')
        node = documentationtools.GraphvizNode(
            attributes={
                label: str(self.preprolated_duration),
                shape: 'box'
            }
            )
        graph.append(node)
        return graph

    @property
    def rtm_format(self):
        r'''RTM format of rhythm tree leaf.

        ::

            >>> rhythmtreetools.RhythmTreeLeaf(1, is_pitched=True).rtm_format
            '1'
            >>> rhythmtreetools.RhythmTreeLeaf(5, is_pitched=False).rtm_format
            '-5'

        Returns string.
        '''
        if self.is_pitched:
            return '{!s}'.format(self.preprolated_duration)
        return '-{!s}'.format(self.preprolated_duration)

    ### PUBLIC PROPERTIES ###

    @property
    def is_pitched(self):
        r'''Gets and sets boolean equal to  true if leaf is pitched.
        Otherwise false.

        ::

            >>> leaf = rhythmtreetools.RhythmTreeLeaf()
            >>> leaf.is_pitched
            True

        ::

            >>> leaf.is_pitched = False
            >>> leaf.is_pitched
            False

        Returns boolean.
        '''
        return self._is_pitched

    @is_pitched.setter
    def is_pitched(self, arg):
        self._is_pitched = bool(arg)


# -*- encoding: utf-8 -*-
import numbers
from abjad.tools.pitchtools.PitchClass import PitchClass


class NamedPitchClass(PitchClass):
    '''A named pitch-class.

    ::

        >>> pitchtools.NamedPitchClass('cs')
        NamedPitchClass('cs')

    ::

        >>> pitchtools.NamedPitchClass(14)
        NamedPitchClass('d')

    ::

        >>> pitchtools.NamedPitchClass(NamedPitch('g,'))
        NamedPitchClass('g')

    ::

        >>> pitchtools.NamedPitchClass(pitchtools.NumberedPitch(15))
        NamedPitchClass('ef')

    ::

        >>> pitchtools.NamedPitchClass(pitchtools.NumberedPitchClass(4))
        NamedPitchClass('e')

    ::

        >>> pitchtools.NamedPitchClass('C#5')
        NamedPitchClass('cs')

    ::

        >>> pitchtools.NamedPitchClass(Note("a'8."))
        NamedPitchClass('a')

    ::

        >>> pitch_class = pitchtools.NamedPitchClass('cs')

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_alteration_in_semitones',
        '_diatonic_pitch_class_number',
        )

    ### INITIALIZER ###

    def __init__(self, expr=None):
        from abjad.tools import pitchtools
        if isinstance(expr, type(self)):
            self._initialize_by_named_pitch_class(expr)
        elif isinstance(expr, pitchtools.NamedPitch):
            self._initialize_by_named_pitch(expr)
        elif pitchtools.Pitch.is_pitch_class_octave_number_string(expr):
            self._initialize_by_pitch_class_octave_number_string(expr)
        elif pitchtools.Pitch.is_pitch_name(expr):
            self._initialize_by_pitch_name(expr)
        elif isinstance(expr, (
            numbers.Number,
            pitchtools.NumberedPitch,
            pitchtools.NumberedPitchClass,
            )):
            self._initialize_by_number(float(expr))
        elif pitchtools.Pitch.is_pitch_carrier(expr):
            self._initialize_by_pitch_carrier(expr)
        elif expr is None:
            self._initialize_by_number(0)
        else:
            message = 'can not instantiate {} from {!r}.'
            message = message.format(type(self).__name__, expr)
            raise TypeError(message)

    ### SPECIAL METHODS ###

    def __add__(self, named_interval):
        r'''Adds `named_interval` to named pitch-class.

        ::

            >>> pitch_class + pitchtools.NamedInterval('+M9')
            NamedPitchClass('ds')

        Return new named pitch-class.
        '''
        from abjad.tools import pitchtools
        dummy = pitchtools.NamedPitch(
            self.pitch_class_name, 4)
        mdi = named_interval
        new = pitchtools.transpose_pitch_carrier_by_interval(
            dummy, mdi)
        return type(self)(new)

    def __copy__(self, *args):
        r'''Copies named pitch-class.

        ::

            >>> import copy
            >>> copy.copy(pitch_class)
            NamedPitchClass('cs')

        Returns new named pitch-class.
        '''
        return type(self)(self)

    def __eq__(self, expr):
        r'''Is true when `expr` can be coerced to a named pitch-class with
        pitch-class name equal to that of this named pitch-class.

        ::

            >>> pitch_class == 'cs'
            True

        Otherwise false:

            >>> pitch_class == 'ds'
            False

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self.pitch_class_name == \
                expr.pitch_class_name
        return self.pitch_class_name == expr

    def __float__(self):
        r'''Changes named pitch-class to a float.

        ::

            >>> float(pitch_class)
            1.0

        Returns float.
        '''
        return float(self.numbered_pitch_class)

    def __hash__(self):
        r'''Hashes named pitch-class.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NamedPitchClass, self).__hash__()

    def __int__(self):
        r'''Changes named pitch-class to an integer.

        ::

            >>> int(pitch_class)
            1

        Returns nonnegative integer.
        '''
        return int(self.numbered_pitch_class)

    def __str__(self):
        r'''String representation of named pitch-class.

        ::

            >>> str(pitch_class)
            'cs'

        Returns string.
        '''
        return self.pitch_class_name

    def __sub__(self, arg):
        r'''Subtracts `arg` from named pitch-class.

        ::

            >>> pitch_class - pitchtools.NamedPitchClass('g')
            NamedInversionEquivalentIntervalClass('+aug4')

        Returns named inversion-equivalent interval-class.
        '''
        if not isinstance(arg, type(self)):
            message = 'must be named pitch-class: {!r}.'
            message = message.format(arg)
            raise TypeError(message)
        from abjad.tools import pitchtools
        pitch_1 = pitchtools.NamedPitch(self, 4)
        pitch_2 = pitchtools.NamedPitch(arg, 4)
        mdi = pitchtools.NamedInterval.from_pitch_carriers(
            pitch_1, pitch_2)
        dic = pitchtools.NamedInversionEquivalentIntervalClass(
            mdi.quality_string, mdi.number)
        return dic

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.pitch_class_name,
            )
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    def _initialize_by_named_pitch(self, expr):
        self._alteration_in_semitones = expr.alteration_in_semitones
        self._diatonic_pitch_class_number = expr.diatonic_pitch_class_number

    def _initialize_by_named_pitch_class(self, expr):
        self._alteration_in_semitones = expr.alteration_in_semitones
        self._diatonic_pitch_class_number = expr.diatonic_pitch_class_number

    def _initialize_by_number(self, expr):
        from abjad.tools import pitchtools
        pitch_class_number = float(expr) % 12
        numbered_pitch_class = pitchtools.NumberedPitchClass(
            pitch_class_number)
        pitch_class_name = numbered_pitch_class.pitch_class_name
        self._initialize_by_pitch_name(pitch_class_name)

    def _initialize_by_pitch_carrier(self, expr):
        from abjad.tools import pitchtools
        named_pitch = pitchtools.get_named_pitch_from_pitch_carrier(
            expr)
        self._initialize_by_named_pitch(named_pitch)

    def _initialize_by_pitch_class_octave_number_string(self, expr):
        from abjad.tools import pitchtools
        groups = pitchtools.Pitch._pitch_class_octave_number_regex.match(
            expr).groups()
        diatonic_pitch_class_name = groups[0].lower()
        symbolic_string = groups[1]
        self._alteration_in_semitones = \
            pitchtools.Accidental._symbolic_string_to_semitones[
                symbolic_string]
        self._diatonic_pitch_class_number = \
            self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
                diatonic_pitch_class_name]

    def _initialize_by_pitch_name(self, expr):
        from abjad.tools import pitchtools
        match = pitchtools.Pitch._pitch_name_regex.match(expr.lower())
        if match is None:
            raise ValueError
        groups = match.groups()
        diatonic_pitch_class_name = groups[0]
        abbreviation = groups[1]
        self._alteration_in_semitones = \
            pitchtools.Accidental._abbreviation_to_semitones[abbreviation]
        self._diatonic_pitch_class_number = \
            self._diatonic_pitch_class_name_to_diatonic_pitch_class_number[
                diatonic_pitch_class_name]

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental):
        r'''Applies `accidental` to named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').apply_accidental('qs')
            NamedPitchClass('ctqs')

        Returns new named pitch-class.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        new_accidental = self.accidental + accidental
        new_name = self.diatonic_pitch_class_name + new_accidental.abbreviation
        return type(self)(new_name)

    def invert(self):
        r'''Inverts named pitch-class.

        Not yet implemented.
        '''
        raise NotImplementedError

    def multiply(self, n=1):
        r'''Multiplies named pitch-class by `n`.

        ::

            >>> pitchtools.NamedPitchClass('cs').multiply(3)
            NamedPitchClass('ef')

        Returns new named pitch-class.
        '''
        return type(self)(self.pitch_class_number * n)

    def transpose(self, expr):
        r'''Transposes named pitch-class by named interval `expr`.

        ::

            >>> named_interval = pitchtools.NamedInterval('major', 2)
            >>> pitchtools.NamedPitchClass('cs').transpose(named_interval)
            NamedPitchClass('ds')

        Returns new named pitch-class.
        '''
        from abjad.tools import pitchtools
        named_interval = pitchtools.NamedInterval(expr)
        pitch = pitchtools.NamedPitch(self, 4)
        transposed_pitch = \
            pitchtools.transpose_pitch_carrier_by_interval(
                pitch, named_interval)
        return type(self)(transposed_pitch)

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Accidental of named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').accidental
            Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self._alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Alteration of named pitch-class in semitones.

        ::

            >>> pitchtools.NamedPitchClass('cs').alteration_in_semitones
            1

        Returns integer or float.
        '''
        return self._alteration_in_semitones

    @property
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name of named interval.

        ::

            >>> pitchtools.NamedPitchClass('cs').diatonic_pitch_class_name
            'c'

        Returns string.
        '''
        return self._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
            self._diatonic_pitch_class_number]

    @property
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number of named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').diatonic_pitch_class_number
            0

        Returns integer.
        '''
        return self._diatonic_pitch_class_number

    @property
    def named_pitch_class(self):
        r'''Named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').named_pitch_class
            NamedPitchClass('cs')

        Returns new named pitch-class.
        '''
        return type(self)(self)

    @property
    def numbered_pitch_class(self):
        r'''Numbered pitch-class corresponding to named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').numbered_pitch_class
            NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def pitch_class_label(self):
        r'''Pitch-class label of named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').pitch_class_label
            'C#'

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            )

    @property
    def pitch_class_name(self):
        r'''Pitch-class name of named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').pitch_class_name
            'cs'

        Returns string.
        '''
        from abjad.tools import pitchtools
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            pitchtools.Accidental._semitones_to_abbreviation[
                self._alteration_in_semitones],
            )

    @property
    def pitch_class_number(self):
        r'''Pitch-class number of named pitch-class.

        ::

            >>> pitchtools.NamedPitchClass('cs').pitch_class_number
            1

        Returns integer or float.
        '''
        return (
            self._diatonic_pitch_class_number_to_pitch_class_number[
                self._diatonic_pitch_class_number] +
                self._alteration_in_semitones) % 12


# -*- encoding: utf-8 -*_


def zip_sequences(sequences, cyclic=False, truncate=True):
    r'''Zips `sequences`.

    ..  container:: example

        Zips two `sequences` cyclically:

        ::

            >>> sequences = [[1, 2, 3], ['a', 'b']]
            >>> sequencetools.zip_sequences(sequences, cyclic=True)
            [(1, 'a'), (2, 'b'), (3, 'a')]

    ..  container:: example

        Zips three `sequences` cyclically:

        ::

            >>> sequences = [[10, 11, 12], [20, 21], [30, 31, 32, 33]]
            >>> sequencetools.zip_sequences(sequences, cyclic=True)
            [(10, 20, 30), (11, 21, 31), (12, 20, 32), (10, 21, 33)]

    ..  container:: example

        Zips sequences without truncation:

        ::

            >>> sequences = ([1, 2, 3, 4], [11, 12, 13], [21, 22, 23])
            >>> sequencetools.zip_sequences(sequences, truncate=False)
            [(1, 11, 21), (2, 12, 22), (3, 13, 23), (4,)]

    ..  container:: example

        Zips sequences noncyclically and with truncation.
        Equivalent to built-in ``zip()``:

        ::

            >>> sequences = ([1, 2, 3, 4], [11, 12, 13], [21, 22, 23])
            >>> sequencetools.zip_sequences(sequences)
            [(1, 11, 21), (2, 12, 22), (3, 13, 23)]

    Returns list.
    '''

    if cyclic:
        max_length = max([len(x) for x in sequences])
        result = []
        for i in range(max_length):
            part = []
            for sequence in sequences:
                index = i % len(sequence)
                element = sequence[index]
                part.append(element)
            part = tuple(part)
            result.append(part)
    elif not truncate:
        result = []
        max_length = max([len(x) for x in sequences])
        for i in range(max_length):
            part = []
            for sequence in sequences:
                try:
                    part.append(sequence[i])
                except IndexError:
                    pass
            result.append(tuple(part))
    elif truncate:
        result = list(zip(*sequences))

    return result



# -*- encoding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import schemetools
from abjad.tools import tonalanalysistools


_allowed_to_be_empty_string = (
    indicatortools.Articulation,
    pitchtools.PitchArray,
    pitchtools.PitchArrayColumn,
    pitchtools.PitchArrayRow,
    pitchtools.Accidental,
    schemetools.SchemeColor,
    tonalanalysistools.ChordSuspension,
    )

classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___str___01(class_):
    r'''All concrete classes have a string representation.

    With the exception of the exception classes. And those classes listed
    explicitly here.
    '''

    if not inspect.isabstract(class_):
        if not issubclass(class_, Exception):
            instance = class_()
            string = str(instance)
            assert string is not None
            if class_ not in _allowed_to_be_empty_string:
                assert string != ''


# -*- encoding: utf-8 -*-
from abjad.tools import sievetools
from abjad.tools.sievetools import ResidueClass
import pytest


def test_sievetools_Sieve_01():
    r'''Boolean operator defaults to OR.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 0), ResidueClass(3, 0)])

    assert sieve.logical_operator == 'or'


def test_sievetools_Sieve_02():
    r'''Logical OR.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 0), ResidueClass(3, 0)])

    assert sieve.get_boolean_train(6) == [1, 0, 1, 1, 1, 0]
    assert sieve.get_congruent_bases(6) == [0, 2, 3, 4, 6]


def test_sievetools_Sieve_03():
    r'''Logical OR.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 1), ResidueClass(3, 0)])

    assert sieve.get_boolean_train(6) == [1, 1, 0, 1, 0, 1]
    assert sieve.get_congruent_bases(6) == [0, 1, 3, 5, 6]


def test_sievetools_Sieve_04():
    r'''Logical AND.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 0), ResidueClass(3, 0)], 'and')

    assert sieve.logical_operator == 'and'
    assert sieve.get_boolean_train(6) == [1, 0, 0, 0, 0, 0]
    assert sieve.get_congruent_bases(6) == [0, 6]


def test_sievetools_Sieve_05():
    r'''Logical AND.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 1), ResidueClass(3, 0)], 'and')

    assert sieve.get_boolean_train(6) == [0, 0, 0, 1, 0, 0]
    assert sieve.get_congruent_bases(6) == [3]


def test_sievetools_Sieve_06():
    r'''Logical XOR.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 0), ResidueClass(3, 0)], 'xor')

    assert sieve.logical_operator == 'xor'
    assert sieve.get_boolean_train(6) == [0, 0, 1, 1, 1, 0]
    assert sieve.get_congruent_bases(6) == [2, 3, 4]


def test_sievetools_Sieve_07():
    r'''Logical XOR.
    '''

    sieve = sievetools.Sieve([ResidueClass(2, 1), ResidueClass(3, 0)], 'xor')

    assert sieve.get_boolean_train(6) == [1, 1, 0, 0, 0, 1]
    assert sieve.get_congruent_bases(6) == [0, 1, 5, 6]


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_measure_01():

    measure = Measure((2, 8), "c'8 d'8")
    labeltools.color_measure(measure, 'red')


    assert systemtools.TestManager.compare(
        measure,
        r'''
        {
            \override Beam #'color = #red
            \override Dots #'color = #red
            \override NoteHead #'color = #red
            \override Staff.TimeSignature #'color = #red
            \override Stem #'color = #red
            \time 2/8
            c'8
            d'8
            \revert Beam #'color
            \revert Dots #'color
            \revert NoteHead #'color
            \revert Staff.TimeSignature #'color
            \revert Stem #'color
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


# -*- encoding: utf-8 -*-
import abc
import copy
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import lilypondfiletools
from abjad.tools import markuptools
from abjad.tools import mathtools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import stringtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject
from abjad.tools.topleveltools import new


class RhythmMaker(AbjadValueObject):
    '''Rhythm-maker abstract base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beam_specifier',
        '_duration_spelling_specifier',
        '_tie_specifier',
        )

    _class_name_abbreviation = 'RM'

    _human_readable_class_name = 'rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        beam_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        ):
        from abjad.tools import rhythmmakertools
        prototype = (rhythmmakertools.BeamSpecifier, type(None))
        assert isinstance(beam_specifier, prototype)
        prototype = (rhythmmakertools.DurationSpellingSpecifier, type(None))
        assert isinstance(duration_spelling_specifier, prototype)
        prototype = (rhythmmakertools.TieSpecifier, type(None))
        assert isinstance(tie_specifier, prototype)
        self._beam_specifier = beam_specifier
        self._duration_spelling_specifier = duration_spelling_specifier
        self._tie_specifier = tie_specifier

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls rhythm-maker.

        Makes music.

        Applies ties specified by tie specifier.

        Checks output type.

        Returns list of selections.
        '''
        from abjad.tools import rhythmmakertools
        duration_pairs = [
            mathtools.NonreducedFraction(x).pair
            for x in divisions
            ]
        seeds = self._to_tuple(seeds)
        selections = self._make_music(duration_pairs, seeds)
        tie_specifier = self.tie_specifier
        if tie_specifier is None:
            tie_specifier = rhythmmakertools.TieSpecifier()
        tie_specifier._make_ties(selections)
        assert isinstance(selections, list), repr(selections)
        assert len(selections), repr(selections)
        prototype = selectiontools.Selection
        assert all(isinstance(x, prototype) for x in selections), repr(
            selections)
        return selections

    def __eq__(self, expr):
        r'''Is true when `expr` is a rhythm-maker with type and public
        properties equal to those of this rhythm-maker. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __format__(self, format_specification=''):
        r'''Formats rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        Defaults `format_specification=None` to
        `format_specification='storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getstate__(self):
        r'''Gets state of rhythm-maker.

        Returns dictionary.
        '''
        if hasattr(self, '__dict__'):
            return vars(self)
        state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                state[slot] = getattr(self, slot, None)
        return state

    def __hash__(self):
        r'''Hashes rhythm-maker.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(RhythmMaker, self).__hash__()

    def __illustrate__(self, divisions=None):
        r'''Illustrates rhythm-maker.

        Defaults `divisions` to ``3/8``, ``4/8``, ``3/16``, ``4/16``.

        Returns LilyPond file.
        '''
        from abjad.tools import rhythmmakertools
        divisions = divisions or [(3, 8), (4, 8), (3, 16), (4, 16)]
        selections = self(divisions)
        lilypond_file = rhythmmakertools.make_lilypond_file(
            selections,
            divisions,
            )
        return lilypond_file

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_tuplets_or_all_are_leaf_selections(expr):
        if all(isinstance(x, scoretools.Tuplet) for x in expr):
            return True
        elif all(RhythmMaker._is_leaf_selection(x) for x in expr):
            return True
        else:
            return False

    @staticmethod
    def _get_rhythmic_staff(lilypond_file):
        score_block = lilypond_file.items[-1]
        score = score_block.items[0]
        rhythmic_staff = score[-1]
        return rhythmic_staff

    @staticmethod
    def _is_leaf_selection(expr):
        if isinstance(expr, selectiontools.Selection):
            return all(isinstance(x, scoretools.Leaf) for x in expr)
        return False

    @abc.abstractmethod
    def _make_music(self, duration_pairs, seeds):
        pass

    def _make_secondary_duration_pairs(
        self,
        duration_pairs,
        split_divisions_by_counts,
        ):
        if not split_divisions_by_counts:
            return duration_pairs[:]
        numerators = [
            duration_pair.numerator
            for duration_pair in duration_pairs
            ]
        secondary_numerators = sequencetools.split_sequence(
            numerators,
            split_divisions_by_counts,
            cyclic=True,
            overhang=True,
            )
        secondary_numerators = \
            sequencetools.flatten_sequence(secondary_numerators)
        denominator = duration_pairs[0].denominator
        secondary_duration_pairs = [
            (n, denominator)
            for n in secondary_numerators
            ]
        return secondary_duration_pairs

    def _make_tuplets(self, duration_pairs, leaf_lists):
        assert len(duration_pairs) == len(leaf_lists)
        tuplets = []
        for duration_pair, leaf_list in zip(duration_pairs, leaf_lists):
            tuplet = scoretools.FixedDurationTuplet(duration_pair, leaf_list)
            tuplets.append(tuplet)
        return tuplets

    def _none_to_trivial_helper(self, expr):
        if expr is None:
            expr = self._trivial_helper
        assert callable(expr)
        return expr

    def _none_to_tuple(self, expr):
        if expr is None:
            expr = ()
        assert isinstance(expr, tuple), expr
        return expr

    @staticmethod
    def _reverse_tuple(expr):
        if expr is not None:
            return tuple(reversed(expr))

    @staticmethod
    def _rotate_tuple(expr, n):
        if expr is not None:
            return tuple(sequencetools.rotate_sequence(expr, n))

    def _scale_taleas(self, duration_pairs, talea_denominator, taleas):
        dummy_duration_pair = (1, talea_denominator)
        duration_pairs.append(dummy_duration_pair)
        Duration = durationtools.Duration
        duration_pairs = Duration.durations_to_nonreduced_fractions(
            duration_pairs)
        dummy_duration_pair = duration_pairs.pop()
        lcd = dummy_duration_pair.denominator
        multiplier = lcd / talea_denominator
        scaled_taleas = []
        for talea in taleas:
            talea = datastructuretools.CyclicTuple(
                [multiplier * x for x in talea],
                )
            scaled_taleas.append(talea)
        result = [duration_pairs, lcd]
        result.extend(scaled_taleas)
        return tuple(result)

    def _sequence_to_ellipsized_string(self, sequence):
        if not sequence:
            return '[]'
        if len(sequence) <= 4:
            result = ', '.join([str(x) for x in sequence])
        else:
            result = ', '.join([str(x) for x in sequence[:4]])
            result += ', ...'
        result = '[${}$]'.format(result)
        return result

    def _to_tuple(self, expr):
        if isinstance(expr, list):
            expr = tuple(expr)
        return expr

    def _trivial_helper(self, talea, seeds):
        if isinstance(seeds, int) and len(talea):
            return sequencetools.rotate_sequence(talea, seeds)
        return talea

    ### PUBLIC PROPERTIES ###

    @property
    def beam_specifier(self):
        r'''Gets beam specifier of rhythm-maker.

        Returns beam specifier or none.
        '''
        return self._beam_specifier

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier of rhythm-maker.

        Returns duration spelling specifier or none.
        '''
        return self._duration_spelling_specifier

    @property
    def tie_specifier(self):
        r'''Gets tie specifier of rhythm-maker.

        Return tie specifier or none.
        '''
        return self._tie_specifier

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses rhythm-maker.

        Concrete rhythm-makers should override this method.

        Returns new rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        specifier = specifier.reverse()
        maker = new(
            self,
            duration_spelling_specifier=specifier,
            )
        return maker


# -*- encoding: utf-8 -*-
import copy
from abjad.tools.abctools.AbjadObject import AbjadObject


class OctaveTranspositionMappingComponent(AbjadObject):
    '''An octave transposition mapping component.

    ::

        >>> mc = pitchtools.OctaveTranspositionMappingComponent('[A0, C8]', 15)
        >>> mc
        OctaveTranspositionMappingComponent(source_pitch_range=PitchRange(range_string='[A0, C8]'), target_octave_start_pitch=NumberedPitch(15))

    Initializes from input parameters separately, from a pair, from
    a string or from another mapping component.

    Models
    ``pitchtools.transpose_pitch_number_by_octave_transposition_mapping``
    input part. (See the docs for that function.)

    Octave transposition mapping components are mutable.

    .. todo:: make components immutable.
    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        source_pitch_range='[A0, C8]', 
        target_octave_start_pitch=0,
        ):
        from abjad.tools import pitchtools
        if isinstance(source_pitch_range, pitchtools.PitchRange):
            source_pitch_range = copy.copy(source_pitch_range)
        else:
            source_pitch_range = pitchtools.PitchRange(source_pitch_range)
        target_octave_start_pitch = pitchtools.NumberedPitch(
            target_octave_start_pitch)
        self.source_pitch_range = source_pitch_range
        self.target_octave_start_pitch = target_octave_start_pitch

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a an octave transposition mapping component
        with source pitch range and target octave start pitch equal to those of
        this octave transposition mapping component. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.source_pitch_range == expr.source_pitch_range:
                if self.target_octave_start_pitch == \
                    expr.target_octave_start_pitch:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats mapping component.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes octave transposition mapping component.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(OctaveTranspositionMappingComponent, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='source_pitch_range',
                command='pr',
                editor=idetools.getters.get_pitch_range_string,
                is_keyword=False,
                ),
            systemtools.AttributeDetail(
                name='target_octave_start_pitch',
                command='sp',
                editor=idetools.getters.get_integer,
                is_keyword=False,
                ),
            )

    @property
    def _input_argument_token(self):
        return '({!r}, {:d})'.format(
            self.source_pitch_range.one_line_named_pitch_repr,
            self.target_octave_start_pitch,
            )

    @property
    def _list_format(self):
        return ((
            self.source_pitch_range.start_pitch.pitch_number,
            self.source_pitch_range.stop_pitch.pitch_number),
            self.target_octave_start_pitch.pitch_number)

    @property
    def _one_line_menu_summary(self):
        return '{} => {:d}'.format(
            self.source_pitch_range.one_line_named_pitch_repr,
            self.target_octave_start_pitch,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def source_pitch_range(self):
        r'''Gets and sets source pitch range of mapping component.

        ::

            >>> mc.source_pitch_range
            PitchRange(range_string='[A0, C8]')

        Returns pitch range or none.
        '''
        return self._source_pitch_range

    @source_pitch_range.setter
    def source_pitch_range(self, source_pitch_range):
        from abjad.tools import pitchtools
        if isinstance(source_pitch_range, str):
            source_pitch_range = pitchtools.PitchRange(source_pitch_range)
        elif isinstance(source_pitch_range, pitchtools.PitchRange):
            source_pitch_range = copy.copy(source_pitch_range)
        self._source_pitch_range = source_pitch_range

    @property
    def target_octave_start_pitch(self):
        r'''Gets and sets target octave start pitch of mapping component.

        ::

            >>> mc.target_octave_start_pitch
            NumberedPitch(15)

        Returns numbered pitch or none.
        '''
        return self._target_octave_start_pitch

    @target_octave_start_pitch.setter
    def target_octave_start_pitch(self, target_octave_start_pitch):
        from abjad.tools import pitchtools
        self._target_octave_start_pitch = pitchtools.NumberedPitch(
            target_octave_start_pitch)


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_note_head_by_numbered_pitch_class_color_map_01():

    note = Note(12, (1, 4))
    labeltools.color_note_head_by_numbered_pitch_class_color_map(note)

    assert systemtools.TestManager.compare(
        note,
        r'''
        \once \override NoteHead #'color = #(x11-color 'red)
        c''4
        '''
        )

    assert inspect_(note).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_positive_integers_01():

    assert mathtools.all_are_positive_integers([1, 2, 3, 99])


def test_mathtools_all_are_positive_integers_02():

    assert not mathtools.all_are_positive_integers([0, 2, 3, 99])
    assert not mathtools.all_are_positive_integers([-1, 2, 3, 99])
    assert not mathtools.all_are_positive_integers(7)
    assert not mathtools.all_are_positive_integers('foo')


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class StaffChange(AbjadObject):
    r'''A staff change.

    ::

        >>> staff_group = StaffGroup()
        >>> staff_group.context_name = 'PianoStaff'
        >>> rh_staff = Staff("c'8 d'8 e'8 f'8")
        >>> rh_staff.name = 'RHStaff'
        >>> lh_staff = Staff("s2")
        >>> lh_staff.name = 'LHStaff'
        >>> staff_group.extend([rh_staff, lh_staff])
        >>> staff_change = indicatortools.StaffChange(lh_staff)
        >>> attach(staff_change, rh_staff[2])
        >>> show(staff_group) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff_group))
        \new PianoStaff <<
            \context Staff = "RHStaff" {
                c'8
                d'8
                \change Staff = LHStaff
                e'8
                f'8
            }
            \context Staff = "LHStaff" {
                s2
            }
        >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_staff',
        )

    _format_leaf_children = False

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, staff=None):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        if not isinstance(staff, (scoretools.Staff, type(None))):
            message = 'staff change input value {!r}'
            message += ' must be staff instance.'
            message.format(staff)
            raise TypeError(message)
        self._staff = staff

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies staff change.

        Returns new staff change.
        '''
        return type(self)(self.staff)

    def __eq__(self, expr):
        r'''Is true when `expr` is a staff change with a staff value equal
        to that of this staff change. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self.staff is expr.staff
        return False

    def __hash__(self):
        r'''Hashes staff change.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(StaffChange, self).__hash__()

    def __str__(self):
        r'''Gets string format of staff change.

        Returns string.
        '''
        from abjad.tools import schemetools
        if self.staff is None:
            return r'\change Staff = ##f'
        return r'\change Staff = {}'.format(
            schemetools.Scheme.format_scheme_value(self.staff.name),
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.staff)

    @property
    def _lilypond_format(self):
        return str(self)

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.opening.commands.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def staff(self):
        r'''Staff of staff change.

        ::

            >>> staff_change.staff
            Staff('s2')

        Returns staff.
        '''
        return self._staff



# -*- encoding: utf-8 -*-


def new(expr, **kwargs):
    r'''Makes new `expr` with optional new `kwargs`.

    Returns new object with the same type as `expr`.
    '''
    from abjad.tools import systemtools
    if expr is None:
        return expr
    manager = systemtools.StorageFormatManager
    keyword_argument_dictionary = \
        manager.get_keyword_argument_dictionary(expr)
    positional_argument_dictionary = \
        manager.get_positional_argument_dictionary(expr)
    recursive_arguments = {}
    for key, value in kwargs.items():
        if '__' in key:
            key, divider, subkey = key.partition('__')
            if key not in recursive_arguments:
                recursive_arguments[key] = []
            pair = (subkey, value)
            recursive_arguments[key].append(pair)
            continue
        if key in positional_argument_dictionary:
            positional_argument_dictionary[key] = value
        elif key in keyword_argument_dictionary:
            keyword_argument_dictionary[key] = value
        else:
            raise KeyError(key)
    for key, pairs in recursive_arguments.items():
        recursed_object = getattr(expr, key)
        if recursed_object is None:
            continue
        recursive_keyword_argument_dictionary = dict(pairs)
        recursed_object = new(
            recursed_object,
            **recursive_keyword_argument_dictionary
            )
        if key in positional_argument_dictionary:
            positional_argument_dictionary[key] = recursed_object
        elif key in keyword_argument_dictionary:
            keyword_argument_dictionary[key] = recursed_object
    positional_argument_values = []
    positional_argument_names = getattr(
        expr, '_positional_argument_names', None) or \
        manager.get_positional_argument_names(expr)
    for positional_argument_name in positional_argument_names:
        positional_argument_value = positional_argument_dictionary[
            positional_argument_name]
        positional_argument_values.append(positional_argument_value)
    result = type(expr)(
        *positional_argument_values, **keyword_argument_dictionary)
    return result


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class NonreducedRatio(AbjadObject, tuple):
    '''Nonreduced ratio of one or more nonzero integers.

    Initializes from one or more nonzero integers:

    ::

        >>> mathtools.NonreducedRatio(2, 4, 2)
        NonreducedRatio(2, 4, 2)

    Initializes from a tuple or list:

    ::

        >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
        >>> ratio
        NonreducedRatio(2, 4, 2)

    Uses a tuple to return ratio integers.

        >>> tuple(ratio)
        (2, 4, 2)

    Nonreduced ratios are immutable.
    '''

    ### CONSTRUCTOR ###

    def __new__(cls, *args):
        if len(args) == 1 and isinstance(args[0], (list, tuple)):
            args = args[0]
        elif len(args) == 0:
            args = (1, 1)
        assert args, repr(args)
        assert all(x != 0 for x in args), repr(args)
        self = tuple.__new__(cls, args)
        return self

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a nonreduced ratio with numerator and
        denominator equal to those of this nonreduced ratio. Otherwise false.

        Returns boolean.
        '''
        expr = type(self)(expr)
        return tuple(self) == tuple(expr)

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes non-reduced ratio.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NonreducedRatio, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=tuple(self),
            )


# -*- encoding: utf-8 -*-
import collections
import re
from abjad.tools import mathtools
from abjad.tools import stringtools
from abjad.tools.pitchtools.Pitch import Pitch


class NamedPitch(Pitch):
    '''A named pitch.

    ::

        >>> pitch = NamedPitch("cs''")
        >>> show(pitch) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_alteration_in_semitones',
        '_diatonic_pitch_class_number',
        '_octave_number',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import pitchtools
        if args and \
            isinstance(args[0], collections.Iterable) and \
            not stringtools.is_string(args[0]) and \
            len(args) == 1:
            args = args[0]
        if len(args) == 1:
            if isinstance(args[0], (int, float)):
                arg = mathtools.integer_equivalent_number_to_integer(
                    float(args[0]))
                self._initialize_by_pitch_number(arg)
            elif isinstance(args[0], type(self)):
                self._initialize_by_named_pitch(*args)
            elif isinstance(args[0], pitchtools.NumberedPitch):
                self._initialize_by_pitch_number(
                    args[0].pitch_number)
            elif isinstance(args[0], pitchtools.PitchClass):
                self._initialize_by_named_pitch_class_and_octave_number(
                    pitchtools.NamedPitchClass(args[0]), 4)
            elif hasattr(args[0], 'named_pitch'):
                self._initialize_by_named_pitch(args[0].named_pitch)
            elif self.is_pitch_class_octave_number_string(args[0]):
                self._initialize_by_pitch_class_octave_number_string(*args)
            elif isinstance(args[0], str):
                self._initialize_by_pitch_name(*args)
            else:
                message = 'can not initialize {} from {!r}.'
                message = message.format(type(self).__name__, args)
                raise ValueError(message)
        elif len(args) == 2:
            if isinstance(args[0], str):
                self._initialize_by_pitch_class_name_and_octave_number(*args)
            elif isinstance(args[0], pitchtools.NamedPitchClass):
                self._initialize_by_named_pitch_class_and_octave_number(*args)
            elif isinstance(args[0], (int, float)):
                if isinstance(args[1], str):
                    self._initialize_by_pitch_number_and_diatonic_pitch_class_name(
                        *args)
                elif isinstance(args[1], pitchtools.NamedPitchClass):
                    self._initialize_by_pitch_number_and_named_pitch_class(*args)
                else:
                    raise TypeError
            else:
                message = 'can not initialize {}: {!r}.'
                message = message.format(type(self).__name__, args)
                raise ValueError(message)
        elif len(args) == 0:
            self._initialize_by_pitch_class_name_and_octave_number('c', 4)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __add__(self, interval):
        r'''Adds named pitch to `interval`.

        ::

            >>> pitch + pitchtools.NamedInterval('+M2')
            NamedPitch("ds''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NamedInterval(interval)
        return pitchtools.transpose_pitch_carrier_by_interval(
            self, interval)

    def __copy__(self, *args):
        r'''Copies named pitch.

        ::

            >>> import copy
            >>> copy.copy(pitch)
            NamedPitch("cs''")

        Returns new named pitch.
        '''
        return type(self)(self)

    def __eq__(self, arg):
        r'''Is true when `arg` is a named pitch equal to this named pitch.

        ::

            >>> pitch == NamedPitch("cs''")
            True

        Otherwise false:

        ::

            >>> pitch == NamedPitch("ds''")
            False

        Returns boolean.
        '''
        try:
            arg = type(self)(arg)
            if str(self) == str(arg):
                return True
            return False
        except (TypeError, ValueError):
            return False

    def __float__(self):
        r'''Changes named pitch to float.

        ::

            >>> float(pitch)
            13.0

        Returns float.
        '''
        return float(self.pitch_number)

    def __ge__(self, arg):
        r'''Is true when named pitch is greater than or equal to `arg`.
        Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            return self.diatonic_pitch_number > arg.diatonic_pitch_number or \
                (self.diatonic_pitch_number == arg.diatonic_pitch_number and
                self.alteration_in_semitones >= arg.alteration_in_semitones)
        elif isinstance(arg, pitchtools.PitchRange):
            return self >= arg.stop_pitch
        else:
            try:
                arg = type(self)(arg)
                return self.__ge__(arg)
            except (TypeError, ValueError):
                pass
        return False

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.pitch_name,)

    def __gt__(self, arg):
        r'''Is true when named pitch is greater than `arg`. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            return self.diatonic_pitch_number > arg.diatonic_pitch_number or \
                (self.diatonic_pitch_number == arg.diatonic_pitch_number and \
                self.alteration_in_semitones > arg.alteration_in_semitones)
        elif isinstance(arg, pitchtools.PitchRange):
            return self > arg.stop_pitch
        else:
            try:
                arg = type(self)(arg)
                return self.__gt__(arg)
            except (TypeError, ValueError):
                pass
        return False

    def __hash__(self):
        r'''Required to be explicitely re-defined on Python 3 if
        __eq__ changes.

        Returns integer.
        '''
        return super(NamedPitch, self).__hash__()

    def __int__(self):
        r'''Changes named pitch to integer.

        ::

            >>> int(pitch)
            13

        Returns integer.
        '''
        if not mathtools.is_integer_equivalent_number(self.pitch_number):
            raise TypeError
        return int(self.pitch_number)

    def __le__(self, arg):
        r'''Is true when named pitch is less than or equal to `arg`. Otherwise
        false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            if not self.diatonic_pitch_number == arg.diatonic_pitch_number:
                return self.diatonic_pitch_number <= arg.diatonic_pitch_number
            if not self.alteration_in_semitones == arg.alteration_in_semitones:
                return self.alteration_in_semitones <= \
                    arg.alteration_in_semitones
            return True
        elif isinstance(arg, pitchtools.PitchRange):
            return self <= arg.start_pitch
        else:
            try:
                arg = type(self)(arg)
                return self.__le__(arg)
            except (TypeError, ValueError):
                pass
        return False

    def __lt__(self, arg):
        r'''Is true when named pitch is less than `arg`. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            return self.diatonic_pitch_number < arg.diatonic_pitch_number or \
                (self.diatonic_pitch_number == arg.diatonic_pitch_number and \
                self.alteration_in_semitones < arg.alteration_in_semitones)
        elif isinstance(arg, pitchtools.PitchRange):
            return self < arg.start_pitch
        elif arg is None:
            return True
        else:
            try:
                arg = type(self)(arg)
                return self.__lt__(arg)
            except (TypeError, ValueError):
                pass
        return False

    def __ne__(self, arg):
        r'''Is true when named pitch does not equal `arg`.

        ::

            >>> NamedPitch("cs''") != NamedPitch("ds''")
            True

        Otherwise false:

        ::

            >>> NamedPitch("cs''") != NamedPitch("cs''")
            False

        Returns boolean.
        '''
        return not self == arg

    def __str__(self):
        r'''String representation of named pitch.

        ::

            >>> str(NamedPitch("cs''"))
            "cs''"

        Returns string.
        '''
        return self.pitch_name

    def __sub__(self, arg):
        r'''Subtracts `arg` from named pitch.

        ::

            >>> NamedPitch("cs''") - NamedPitch("b'")
            NamedInterval('-M2')

        Returns named interval.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            return pitchtools.NamedInterval.from_pitch_carriers(
                self, arg)
        else:
            interval = arg
            return pitchtools.transpose_pitch_carrier_by_interval(
                self, -interval)

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=(
                self.pitch_name,
                )
            )

    ### PRIVATE METHODS ###

    def _initialize_by_named_pitch(self, named_pitch):
        self._alteration_in_semitones = named_pitch._alteration_in_semitones
        self._diatonic_pitch_class_number = \
            named_pitch.diatonic_pitch_class_number
        self._octave_number = named_pitch.octave_number

    def _initialize_by_named_pitch_class_and_octave_number(
        self, named_pitch_class, octave_number):
        self._alteration_in_semitones = \
            named_pitch_class._alteration_in_semitones
        self._diatonic_pitch_class_number = \
            named_pitch_class._diatonic_pitch_class_number
        self._octave_number = int(octave_number)

    def _initialize_by_pitch_class_name_and_octave_number(
        self, pitch_class_name, octave_number):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_class_name)
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_class_name_octave_number_pair(self, pair):
        pitch_class_name, octave_number = pair
        self._initialize_by_pitch_class_name_and_octave_number(
            pitch_class_name, octave_number)

    def _initialize_by_pitch_class_octave_number_string(
        self, pitch_class_octave_number_string):
        from abjad.tools import pitchtools
        groups = self._pitch_class_octave_number_regex.match(
            pitch_class_octave_number_string).groups()
        named_pitch_class = pitchtools.NamedPitchClass(
            pitch_class_octave_number_string)
        octave_number = int(groups[2])
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_name(self, pitch_string):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_string)
        octave_number = pitchtools.Octave.from_pitch_name(
            pitch_string).octave_number
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_number(self, pitch_number):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_number)
        octave_number = pitch_number // 12 + 4
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_number_and_diatonic_pitch_class_name(
        self, pitch_number, diatonic_pitch_class_name):
        from abjad.tools import pitchtools
        accidental, octave_number = \
            pitchtools.spell_pitch_number(
                pitch_number, diatonic_pitch_class_name)
        pitch_class_name = diatonic_pitch_class_name + \
            accidental.abbreviation
        named_pitch_class = pitchtools.NamedPitchClass(pitch_class_name)
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_number_and_named_pitch_class(
        self, pitch_number, named_pitch_class):
        diatonic_pitch_class_name = named_pitch_class.diatonic_pitch_class_name
        self._initialize_by_pitch_number_and_diatonic_pitch_class_name(
            pitch_number, diatonic_pitch_class_name)

    @property
    def _lilypond_format(self):
        return str(self)

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        '''Applies `accidental` to named pitch.

        ::

            >>> NamedPitch("cs''").apply_accidental('s')
            NamedPitch("css''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        new_accidental = self.accidental + accidental
        new_name = self.diatonic_pitch_class_name
        new_name += new_accidental.abbreviation
        return type(self)(new_name, self.octave_number)

    def invert(self, axis=None):
        r'''Inverts named pitch around `axis`.

        ::

            >>> pitchtools.NamedPitch("d'").invert("c'")
            NamedPitch('bf')

        ::

            >>> pitchtools.NamedPitch('bf').invert("c'")
            NamedPitch("d'")

        ::

            >>> pitchtools.NamedPitch("d'").invert('a')
            NamedPitch('e')

        Returns new named pitch.
        '''
        return Pitch.invert(self, axis=axis)

    def multiply(self, n=1):
        r'''Multiply pitch-class of named pitch by `n` while maintaining
        octave of named pitch.

        ::

            >>> NamedPitch('d,').multiply(3)
            NamedPitch('fs,')

        Returns new named pitch.
        '''
        pitch_class_number = (self.pitch_class_number * n) % 12
        octave_floor = (self.octave_number - 4) * 12
        return type(self)(pitch_class_number + octave_floor)

    def respell_with_flats(self):
        r'''Respells named pitch with flats.

        ::

            >>> NamedPitch("cs''").respell_with_flats()
            NamedPitch("df''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        octave = pitchtools.Octave.from_pitch_number(
            self.numbered_pitch.pitch_number).octave_number
        name = class_._pitch_class_number_to_pitch_class_name_with_flats[
            self.pitch_class_number]
        pitch = type(self)(name, octave)
        return pitch

    def respell_with_sharps(self):
        r'''Respells named pitch with sharps.

        ::

            >>> NamedPitch("df''").respell_with_sharps()
            NamedPitch("cs''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        octave = pitchtools.Octave.from_pitch_number(
            self.numbered_pitch.pitch_number).octave_number
        name = class_._pitch_class_number_to_pitch_class_name_with_sharps[
            self.pitch_class_number]
        pitch = type(self)(name, octave)
        return pitch

    def transpose(self, expr):
        r'''Transposes named pitch by `expr`.

        ::

            >>> NamedPitch("c'").transpose('m2')
            NamedPitch("df'")

        ::

            >>> NamedPitch("c'").transpose('-M2')
            NamedPitch('bf')

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        named_interval = pitchtools.NamedInterval(expr)
        transposed_pitch = pitchtools.transpose_pitch_carrier_by_interval(
            self, named_interval)
        return type(self)(transposed_pitch)

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Accidental of named pitch.

        ::

            >>> NamedPitch("cs''").accidental
            Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self._alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Alteration of named pitch in semitones.

        ::

            >>> NamedPitch("cs''").alteration_in_semitones
            1

        Returns integer or float.
        '''
        return self._alteration_in_semitones

    @property
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name of named pitch.

        ::

            >>> NamedPitch("cs''").diatonic_pitch_class_name
            'c'

        Returns string.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        return class_._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
            self._diatonic_pitch_class_number]

    @property
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number of named pitch.

        ::

            >>> NamedPitch("cs''").diatonic_pitch_class_number
            0

        Returns integer.
        '''
        return self._diatonic_pitch_class_number

    @property
    def diatonic_pitch_name(self):
        r'''Diatonic pitch name of named pitch.

        ::

            >>> NamedPitch("cs''").diatonic_pitch_name
            "c''"

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            self.octave.octave_tick_string,
            )

    @property
    def diatonic_pitch_number(self):
        r'''Diatonic pitch number of named pitch.

        ::

            >>> NamedPitch("cs''").diatonic_pitch_number
            7

        Returns integer.
        '''
        return ((self._octave_number - 4) * 7) + \
            self._diatonic_pitch_class_number

    @property
    def named_pitch(self):
        r'''Named pitch.

        ::

            >>> NamedPitch("cs''").named_pitch
            NamedPitch("cs''")

        Returns new named pitch.
        '''
        return type(self)(self)

    @property
    def named_pitch_class(self):
        r'''Named pitch-class of named pitch.

        ::

            >>> NamedPitch("cs''").named_pitch_class
            NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def numbered_pitch(self):
        r'''Numbered pitch corresponding to named pitch.

        ::

            >>> NamedPitch("cs''").numbered_pitch
            NumberedPitch(13)

        Returns numbered pitch.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch(self)

    @property
    def numbered_pitch_class(self):
        r'''Numbered pitch-class corresponding to named pitch.

        ::

            >>> NamedPitch("cs''").numbered_pitch_class
            NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def octave(self):
        r'''Octave of named pitch.

        ::

            >>> NamedPitch("cs''").octave
            Octave(5)

        Returns octave.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Octave(self._octave_number)

    @property
    def octave_number(self):
        r'''Integer octave number of named pitch.

        ::

            >>> NamedPitch("cs''").octave_number
            5

        Returns integer.
        '''
        return int(self._octave_number)

    @property
    def pitch_class_name(self):
        r'''Pitch-class name of named pitch.

        ::

            >>> NamedPitch("cs''").pitch_class_name
            'cs'

        Returns string.
        '''
        from abjad.tools import pitchtools
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            pitchtools.Accidental._semitones_to_abbreviation[
                self._alteration_in_semitones],
            )

    @property
    def pitch_class_number(self):
        r'''Pitch-class number of named pitch.

        ::

            >>> NamedPitch("cs''").pitch_class_number
            1

        Returns integer or float.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        return (class_._diatonic_pitch_class_number_to_pitch_class_number[
            self._diatonic_pitch_class_number] + \
            self._alteration_in_semitones) % 12

    @property
    def pitch_class_octave_label(self):
        r'''Pitch-class / octave label of named pitch.

        ::

            >>> NamedPitch("cs''").pitch_class_octave_label
            'C#5'

        Returns string.
        '''
        return '{}{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            self.octave_number,
            )

    @property
    def pitch_name(self):
        r'''Pitch name of named pitch.

        ::

            >>> NamedPitch("cs''").pitch_name
            "cs''"

        Returns string.
        '''
        return '{}{}'.format(
            self.pitch_class_name,
            self.octave.octave_tick_string,
            )

    @property
    def pitch_number(self):
        r'''Pitch-class number of named pitch.

        ::

            >>> NamedPitch("cs''").pitch_number
            13

        ::

            >>> NamedPitch("cff''").pitch_number
            10

        Returns integer or float.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        return class_._diatonic_pitch_class_number_to_pitch_class_number[
            self.diatonic_pitch_class_number] + \
            self.alteration_in_semitones + \
            (12 * (self._octave_number - 4))


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.sievetools import ResidueClass
import pytest


def test_sievetools_ResidueClass_operator_and_01():
    r'''Residue class AND residue class returns a sieve.
    '''

    rc1 = ResidueClass(4, 0)
    rc2 = ResidueClass(4, 1)
    t = rc1 & rc2

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'and'
    assert t.rcs == [rc1, rc2]
    assert t.get_boolean_train(4) == [0,0,0,0]
    assert t.get_congruent_bases(6) == []


def test_sievetools_ResidueClass_operator_and_02():
    r'''and-sieve AND residue class returns a flat and-sieve.
    '''

    rcexpression = ResidueClass(4, 0) & ResidueClass(4, 1)
    rc = ResidueClass(3, 0)
    t = rc & rcexpression

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'and'
    assert len(t.rcs) == 3
    assert rcexpression.rcs[0] in t.rcs
    assert rcexpression.rcs[1] in t.rcs
    assert rc in t.rcs


def test_sievetools_ResidueClass_operator_and_03():
    r'''residue class AND and-sieve returns a flat and-sieve.
    '''

    rcexpression = ResidueClass(4, 0) & ResidueClass(4, 1)
    rc = ResidueClass(3, 0)
    t = rcexpression & rc

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'and'
    assert len(t.rcs) == 3
    assert rcexpression.rcs[0] in t.rcs
    assert rcexpression.rcs[1] in t.rcs
    assert rc in t.rcs


def test_sievetools_ResidueClass_operator_and_04():
    r'''and-sieve AND and-sieve returns a flat and-sieve.
    '''

    rc1 = ResidueClass(4, 0)
    rc2 = ResidueClass(4, 1)
    rc3 = ResidueClass(3, 0)
    rc4 = ResidueClass(3, 1)
    rcsA = rc1 & rc2
    rcsB = rc3 & rc4
    t = rcsA & rcsB

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'and'
    assert len(t.rcs) == 4
    assert rc1 in t.rcs
    assert rc2 in t.rcs
    assert rc3 in t.rcs
    assert rc4 in t.rcs


def test_sievetools_ResidueClass_operator_and_05():
    r'''AND.
    '''

    residueclass = ResidueClass(2, 0) & ResidueClass(3, 0)

    assert isinstance(residueclass, sievetools.Sieve)
    assert residueclass.logical_operator == 'and'
    assert residueclass.get_boolean_train(6) == [1,0,0,0,0,0]
    assert residueclass.get_congruent_bases(6) == [0, 6]


def test_sievetools_ResidueClass_operator_and_06():
    r'''AND.
    '''

    residueclass = ResidueClass(2, 1) & ResidueClass(3, 0)

    assert residueclass.get_boolean_train(6) == [0,0,0,1,0,0]
    assert residueclass.get_congruent_bases(6) == [3]


# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.markuptools.MarkupCommand import MarkupCommand


class MusicGlyph(MarkupCommand):
    r'''A LilyPond music glyph.

    ::

        >>> markuptools.MusicGlyph('accidentals.sharp')
        MusicGlyph('accidentals.sharp')
        >>> print(_)
        \musicglyph #"accidentals.sharp"

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, glyph_name=None):
        from abjad.ly import music_glyphs
        glyph_name = glyph_name or 'accidentals.sharp'
        message = 'not a valid LilyPond glyph name.'
        assert glyph_name in music_glyphs, message
        glyph_scheme = schemetools.Scheme(glyph_name, force_quotes=True)
        MarkupCommand.__init__(self, 'musicglyph', glyph_scheme)

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.args[0]._value,
            )
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )



# -*- encoding: utf-8 -*-
import itertools
from abjad.tools import mathtools


def partition_sequence_by_sign_of_elements(sequence, sign=(-1, 0, 1)):
    '''Partitions `sequence` elements by sign.

    ::

        >>> sequence = [0, 0, -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence))
        [[0, 0], [-1, -1], [2, 3], [-5], [1, 2, 5], [-5, -6]]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[-1]))
        [0, 0, [-1, -1], 2, 3, [-5], 1, 2, 5, [-5, -6]]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[0]))
        [[0, 0], -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[1]))
        [0, 0, -1, -1, [2, 3], -5, [1, 2, 5], -5, -6]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[-1, 0]))
        [[0, 0], [-1, -1], 2, 3, [-5], 1, 2, 5, [-5, -6]]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[-1, 1]))
        [0, 0, [-1, -1], [2, 3], [-5], [1, 2, 5], [-5, -6]]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[0, 1]))
        [[0, 0], -1, -1, [2, 3], -5, [1, 2, 5], -5, -6]

    ::

        >>> list(sequencetools.partition_sequence_by_sign_of_elements(
        ...     sequence, sign=[-1, 0, 1]))
        [[0, 0], [-1, -1], [2, 3], [-5], [1, 2, 5], [-5, -6]]

    When ``-1`` in ``sign``, groups negative elements.

    When ``0`` in ``sign``, groups ``0`` elements.

    When ``1`` in ``sign``, groups positive elements.

    Returns list of tuples of `sequence` element references.
    '''
    result = []
    g = itertools.groupby(sequence, mathtools.sign)
    for current_sign, group in g:
        if current_sign in sign:
            result.append(type(sequence)(group))
        else:
            for x in group:
                result.append(x)
    return result


# -*- encoding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___repr___01(class_):
    r'''All concrete classes have an interpreter representation.
    '''

    if not inspect.isabstract(class_):
        instance = class_()
        string = repr(instance)
        assert string is not None
        assert string != ''


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class BendAfter(AbjadObject):
    r'''A fall or doit.

    ::

        >>> note = Note("c'4")
        >>> bend = indicatortools.BendAfter(-4)
        >>> attach(bend, note)
        >>> show(note) # doctest: +SKIP

    ..  doctest::

        >>> print(format(note))
        c'4 - \bendAfter #'-4.0

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_bend_amount',
        )


    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self, bend_amount=-4):
        if isinstance(bend_amount, type(self)):
            bend_amount = bend_amount.bend_amount
        bend_amount = float(bend_amount)
        self._bend_amount = bend_amount

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies bend after.

        Returns new bend after.
        '''
        return type(self)(self.bend_amount)

    def __eq__(self, expr):
        r'''Is true when `expr` is a bend after indication with bend amount
        equal to that of this bend after indication after. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.bend_amount == expr.bend_amount:
                return True
        return False

    def __hash__(self):
        r'''Hashes bend after.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(BendAfter, self).__hash__()

    def __str__(self):
        r'''String representation of bend after.

        Returns string.
        '''
        return r"- \bendAfter #'{}".format(self.bend_amount)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self.bend_amount)

    @property
    def _lilypond_format(self):
        return str(self)

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def bend_amount(self):
        r'''Amount of bend after.

        ::

            >>> bend.bend_amount
            -4.0

        Returns float.
        '''
        return self._bend_amount



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner___getitem___01():
    r'''Get at nonnegative index in spanner.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    assert beam[0] is voice[0]


def test_spannertools_Spanner___getitem___02():
    r'''Get at negative index in spanner.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    assert beam[-1] is voice[-1]


def test_spannertools_Spanner___getitem___03():
    r'''Get slice from spanner.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    assert beam[-2:] == voice[-2:]


def test_spannertools_Spanner___getitem___04():
    r'''Get all spanner components.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    assert beam[:] == voice[:]


# -*- encoding: utf-8 -*-


def snake_case_to_upper_camel_case(string):
    '''Changes underscore-delimited lowercase `string` to uppercamelcase.

    ..  container:: example

        ::

            >>> string = 'bass_figure_alignment_positioning'
            >>> stringtools.snake_case_to_upper_camel_case(string)
            'BassFigureAlignmentPositioning'

    Returns string.
    '''

    parts = string.split('_')
    parts = [part.title() for part in parts]
    return ''.join(parts)


# -*- encoding: utf-8 -*-
from __future__ import print_function
import sys
from abjad.tools.abctools import ContextManager


class ProgressIndicator(ContextManager):
    r'''A context manager for printing progress indications.
    '''

    ### INITIALIZER ###

    def __init__(self, message=''):
        self._message = message
        self._progress = 0

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters progress indicator.
        '''
        print('{}: {}'.format(self._message, self._progress), end=' ')
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits progress indicator.
        '''
        print()

    def __repr__(self):
        r'''Gets interpreter representation of context manager.

        ..  container:: example

            ::

                >>> context_manager = systemtools.ProgressIndicator()
                >>> context_manager
                <ProgressIndicator()>

        Returns string.
        '''
        return '<{}()>'.format(type(self).__name__)

    ### PUBLIC PROPERTIES ###

    @property
    def message(self):
        r'''Gets message of progress indicator.

        Returns string.
        '''
        return self._message

    ### PUBLIC METHODS ###

    def advance(self):
        r'''Advances the progress indicator's progress count.  Overwrites
        the current terminal line with the progress indicators message and new
        count.
        '''
        self._progress += 1
        sys.stdout.flush()
        print('\r', end=' ')
        print('{}: {}'.format(self._message, self._progress), end=' ')



# -*- encoding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.topleveltools import attach


class StringOrchestraScoreTemplate(AbjadObject):
    '''String orchestra score template.

    ::

        >>> template = templatetools.StringOrchestraScoreTemplate(
        ...     violin_count=6,
        ...     viola_count=4,
        ...     cello_count=3,
        ...     contrabass_count=2,
        ...     )
        >>> score = template()

    ::

        >>> score
        <Score-"String Orchestra Score"<<4>>>

    ..  doctest::

        >>> print(format(score))
        \context Score = "String Orchestra Score" <<
            \context StaffGroup = "Violin Staff Group" <<
                \context Staff = "Violin 1 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 1 }
                    \set Staff.shortInstrumentName = \markup { Vln. 1 }
                    \context Voice = "Violin 1 Voice" {
                    }
                }
                \context Staff = "Violin 2 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 2 }
                    \set Staff.shortInstrumentName = \markup { Vln. 2 }
                    \context Voice = "Violin 2 Voice" {
                    }
                }
                \context Staff = "Violin 3 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 3 }
                    \set Staff.shortInstrumentName = \markup { Vln. 3 }
                    \context Voice = "Violin 3 Voice" {
                    }
                }
                \context Staff = "Violin 4 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 4 }
                    \set Staff.shortInstrumentName = \markup { Vln. 4 }
                    \context Voice = "Violin 4 Voice" {
                    }
                }
                \context Staff = "Violin 5 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 5 }
                    \set Staff.shortInstrumentName = \markup { Vln. 5 }
                    \context Voice = "Violin 5 Voice" {
                    }
                }
                \context Staff = "Violin 6 Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 6 }
                    \set Staff.shortInstrumentName = \markup { Vln. 6 }
                    \context Voice = "Violin 6 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Viola Staff Group" <<
                \context Staff = "Viola 1 Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 1 }
                    \set Staff.shortInstrumentName = \markup { Vla. 1 }
                    \context Voice = "Viola 1 Voice" {
                    }
                }
                \context Staff = "Viola 2 Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 2 }
                    \set Staff.shortInstrumentName = \markup { Vla. 2 }
                    \context Voice = "Viola 2 Voice" {
                    }
                }
                \context Staff = "Viola 3 Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 3 }
                    \set Staff.shortInstrumentName = \markup { Vla. 3 }
                    \context Voice = "Viola 3 Voice" {
                    }
                }
                \context Staff = "Viola 4 Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 4 }
                    \set Staff.shortInstrumentName = \markup { Vla. 4 }
                    \context Voice = "Viola 4 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Cello Staff Group" <<
                \context Staff = "Cello 1 Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 1 }
                    \set Staff.shortInstrumentName = \markup { Vc. 1 }
                    \context Voice = "Cello 1 Voice" {
                    }
                }
                \context Staff = "Cello 2 Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 2 }
                    \set Staff.shortInstrumentName = \markup { Vc. 2 }
                    \context Voice = "Cello 2 Voice" {
                    }
                }
                \context Staff = "Cello 3 Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 3 }
                    \set Staff.shortInstrumentName = \markup { Vc. 3 }
                    \context Voice = "Cello 3 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Contrabass Staff Group" <<
                \context Staff = "Contrabass 1 Staff" {
                    \clef "bass_8"
                    \set Staff.instrumentName = \markup { Contrabass 1 }
                    \set Staff.shortInstrumentName = \markup { Cb. 1 }
                    \context Voice = "Contrabass 1 Voice" {
                    }
                }
                \context Staff = "Contrabass 2 Staff" {
                    \clef "bass_8"
                    \set Staff.instrumentName = \markup { Contrabass 2 }
                    \set Staff.shortInstrumentName = \markup { Cb. 2 }
                    \context Voice = "Contrabass 2 Voice" {
                    }
                }
            >>
        >>

    Returns score template.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        violin_count=6,
        viola_count=4,
        cello_count=3,
        contrabass_count=2,
        ):
        assert 0 <= violin_count
        assert 0 <= viola_count
        assert 0 <= cello_count
        assert 0 <= contrabass_count
        self._violin_count = int(violin_count)
        self._viola_count = int(viola_count)
        self._cello_count = int(cello_count)
        self._contrabass_count = int(contrabass_count)

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls string orchestra template.

        Returns score.
        '''

        string_orchestra_score = scoretools.Score(
            name='String Orchestra Score',
            )

        if self.violin_count:
            violin_staff_group = scoretools.StaffGroup(
                name='Violin Staff Group',
                )
            for i in range(1, self.violin_count + 1):
                violin_voice = scoretools.Voice(
                    name='Violin {} Voice'.format(i),
                    )
                violin_staff = scoretools.Staff(
                    [violin_voice], name='Violin {} Staff'.format(i))
                clef = indicatortools.Clef('treble')
                attach(clef, violin_staff)
                violin = instrumenttools.Violin(
                    instrument_name_markup='Violin {}'.format(i),
                    short_instrument_name_markup='Vln. {}'.format(i),
                    )
                attach(violin, violin_staff)
                violin_staff_group.append(violin_staff)
            string_orchestra_score.append(violin_staff_group)

        if self.viola_count:
            viola_staff_group = scoretools.StaffGroup(
                name='Viola Staff Group',
                )
            for i in range(1, self.viola_count + 1):
                viola_voice = scoretools.Voice(
                    name='Viola {} Voice'.format(i),
                    )
                viola_staff = scoretools.Staff(
                    [viola_voice], name='Viola {} Staff'.format(i))
                clef = indicatortools.Clef('alto')
                attach(clef, viola_staff)
                viola = instrumenttools.Viola(
                    instrument_name_markup='Viola {}'.format(i),
                    short_instrument_name_markup='Vla. {}'.format(i),
                    )
                attach(viola, viola_staff)
                viola_staff_group.append(viola_staff)
            string_orchestra_score.append(viola_staff_group)

        if self.cello_count:
            cello_staff_group = scoretools.StaffGroup(
                name='Cello Staff Group',
                )
            for i in range(1, self.cello_count + 1):
                cello_voice = scoretools.Voice(
                    name='Cello {} Voice'.format(i),
                    )
                cello_staff = scoretools.Staff(
                    [cello_voice], name='Cello {} Staff'.format(i))
                clef = indicatortools.Clef('bass')
                attach(clef, cello_staff)
                cello = instrumenttools.Cello(
                    instrument_name_markup='Cello {}'.format(i),
                    short_instrument_name_markup='Vc. {}'.format(i),
                    )
                attach(cello, cello_staff)
                cello_staff_group.append(cello_staff)
            string_orchestra_score.append(cello_staff_group)

        if self.contrabass_count:
            contrabass_staff_group = scoretools.StaffGroup(
                name='Contrabass Staff Group',
                )
            for i in range(1, self.contrabass_count + 1):
                contrabass_voice = scoretools.Voice(
                    name='Contrabass {} Voice'.format(i),
                    )
                contrabass_staff = scoretools.Staff(
                    [contrabass_voice], name='Contrabass {} Staff'.format(i))
                clef = indicatortools.Clef('bass_8')
                attach(clef, contrabass_staff)
                contrabass = instrumenttools.Contrabass(
                    instrument_name_markup='Contrabass {}'.format(i),
                    short_instrument_name_markup='Cb. {}'.format(i),
                    )
                attach(contrabass, contrabass_staff)
                contrabass_staff_group.append(contrabass_staff)
            string_orchestra_score.append(contrabass_staff_group)

        # return string quartet score
        return string_orchestra_score

    ### PUBLIC PROPERTIES ###

    @property
    def cello_count(self):
        r'''Number of cellos in string orchestra.

        Returns nonnegative integer.
        '''
        return self._cello_count

    @property
    def contrabass_count(self):
        r'''Number of contrabasses in string orchestra.

        Returns nonnegative integer.
        '''
        return self._contrabass_count

    @property
    def viola_count(self):
        r'''Number of violas in string orcestra.

        Returns nonnegative integer.
        '''
        return self._viola_count

    @property
    def violin_count(self):
        r'''Number of violins in string orchestra.

        Returns nonnegative integer.
        '''
        return self._violin_count



# -*- encoding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_Scale_make_score_01():

    scale = tonalanalysistools.Scale('E', 'major')
    score = scale.make_score()

    r'''
    \new Score \with {
        tempoWholesPerMinute = #(ly:make-moment 30 1)
    } <<
        \new Staff {
            \key e \major
            e'8
            fs'8
            gs'8
            a'8
            b'8
            cs''8
            ds''8
            e''8
            ds''8
            cs''8
            b'8
            a'8
            gs'8
            fs'8
            e'4
        }
    >>
    '''

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score \with {
            tempoWholesPerMinute = #(ly:make-moment 30 1)
        } <<
            \new Staff {
                \key e \major
                e'8
                fs'8
                gs'8
                a'8
                b'8
                cs''8
                ds''8
                e''8
                ds''8
                cs''8
                b'8
                a'8
                gs'8
                fs'8
                e'4
            }
        >>
        '''
        )


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools import pitchtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_numbered_interval_classes(
    expr, markup_direction=Up):
    r"""Label leaves in `expr` with numbered interval classes:

    ::

        >>> notes = scoretools.make_notes(
        ...     [0, 25, 11, -4, -14, -13, 9, 10, 6, 5],
        ...     [Duration(1, 8)],
        ...     )
        >>> staff = Staff(notes)
        >>> labeltools.label_leaves_in_expr_with_numbered_interval_classes(
        ...     staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8 ^ \markup { +1 }
            cs'''8 ^ \markup { -2 }
            b'8 ^ \markup { -3 }
            af8 ^ \markup { -10 }
            bf,8 ^ \markup { +1 }
            b,8 ^ \markup { +10 }
            a'8 ^ \markup { +1 }
            bf'8 ^ \markup { -4 }
            fs'8 ^ \markup { -1 }
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    """

    for note in iterate(expr).by_class(scoretools.Note):
        logical_voice_iterator = iterate(note).by_logical_voice_from_component(
            scoretools.Leaf,
            )
        try:
            next(logical_voice_iterator)
            next_leaf = next(logical_voice_iterator)
            if isinstance(next_leaf, scoretools.Note):
                mdi = note.written_pitch - next_leaf.written_pitch
                mci = pitchtools.NumberedInterval(mdi)
                mcic = pitchtools.NumberedIntervalClass(mci)
                markup = markuptools.Markup(mcic, markup_direction)
                attach(markup, note)
        except StopIteration:
            pass



# -*- encoding: utf-8 -*-


def factors(n):
    r'''Integer factors of positive integer `n` in increasing order.

    ::

        >>> mathtools.factors(84)
        [1, 2, 2, 3, 7]

    ::

        >>> for n in range(10, 20):
        ...   print(n, mathtools.factors(n))
        ... 
        10 [1, 2, 5]
        11 [1, 11]
        12 [1, 2, 2, 3]
        13 [1, 13]
        14 [1, 2, 7]
        15 [1, 3, 5]
        16 [1, 2, 2, 2, 2]
        17 [1, 17]
        18 [1, 2, 3, 3]
        19 [1, 19]

    Raises type error on noninteger `n`.

    Raises value error on nonpositive `n`.

    Returns list of one or more positive integers.
    '''
    from abjad.tools import mathtools

    if not mathtools.is_positive_integer(n):
        message = 'must be positive integer: {!r}.'
        message = message.format(n)
        raise TypeError(message)

    d = 2
    factors = [1]
    while 1 < n:
        if n % d == 0:
            factors.append(d)
            n = n/d
        else:
            d = d + 1
    return factors



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_suggest_clef_for_named_pitches_01():

    pitches = [10, 20, 30]
    pitches = [NamedPitch(x) for x in pitches]

    assert pitchtools.suggest_clef_for_named_pitches(pitches) == Clef('treble')


def test_pitchtools_suggest_clef_for_named_pitches_02():

    pitches = [-10, -20, -30]
    pitches = [NamedPitch(x) for x in pitches]

    assert pitchtools.suggest_clef_for_named_pitches(pitches) == Clef('bass')


def test_pitchtools_suggest_clef_for_named_pitches_03():

    pitches = [10, 20, -30]
    pitches = [NamedPitch(x) for x in pitches]

    assert pitchtools.suggest_clef_for_named_pitches(pitches) == Clef('bass')


def test_pitchtools_suggest_clef_for_named_pitches_04():

    pitches = [-10, -20, 30]
    pitches = [NamedPitch(x) for x in pitches]

    assert pitchtools.suggest_clef_for_named_pitches(pitches) == Clef('treble')


def test_pitchtools_suggest_clef_for_named_pitches_05():
    r'''Works with arbitrary expression.
    '''

    staff = Staff(scoretools.make_notes(list(range(-12, -6)), [(1, 4)]))

    assert pitchtools.suggest_clef_for_named_pitches(staff) == Clef('bass')



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_chord_note_heads_in_expr_by_pitch_class_color_map_01():

    chord = Chord([12, 14, 18, 21, 23], (1, 4))
    pitches = [[-12, -10, 4], [-2, 8, 11, 17], [19, 27, 30, 33, 37]]
    colors = ['red', 'blue', 'green']
    color_map = pitchtools.NumberedPitchClassColorMap(pitches, colors)
    labeltools.color_chord_note_heads_in_expr_by_pitch_class_color_map(
        chord, color_map)

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <
            \tweak #'color #red
            c''
            \tweak #'color #red
            d''
            \tweak #'color #green
            fs''
            \tweak #'color #green
            a''
            \tweak #'color #blue
            b''
        >4
        '''
        )

    assert inspect_(chord).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class AbjadValueObject(AbjadObject):
    r'''Abstract base class for classes which compare equally based on their
    storage format.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when all initialization values of Abjad value object equal
        the initialization values of `expr`.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __hash__(self):
        r'''Hashes Abjad value object.

        Returns integer.
        '''
        from abjad.tools import systemtools
        return hash(systemtools.StorageFormatManager.get_hash_values(self))


# -*- encoding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools import datastructuretools
from abjad.tools import mathtools


# load constants into __builtins__ namespace
__builtins__['Less'] = datastructuretools.OrdinalConstant('value', -1, 'Less')
__builtins__['More'] = datastructuretools.OrdinalConstant('value', 1, 'More')
__builtins__['Exact'] = datastructuretools.OrdinalConstant(
    'value', 0, 'Exact')
__builtins__['Left'] = datastructuretools.OrdinalConstant('x', -1, 'Left')
__builtins__['Right'] = datastructuretools.OrdinalConstant('x', 1, 'Right')
__builtins__['Center'] = datastructuretools.OrdinalConstant('y', 0, 'Center')
__builtins__['Up'] = datastructuretools.OrdinalConstant('y', 1, 'Up')
__builtins__['Down'] = datastructuretools.OrdinalConstant('y', -1, 'Down')
__builtins__['Infinity'] = mathtools.Infinity()
__builtins__['NegativeInfinity'] = mathtools.NegativeInfinity()

systemtools.ImportManager.import_structured_package(
    __path__[0],
    globals(),
    delete_systemtools=False,
    )


# -*- encoding: utf-8 -*-


def is_integer_pair(expr):
    r'''Is true when `expr` is an integer tuple of length ``2``.

    ::

        >>> mathtools.is_integer_pair((19, 20))
        True

    Otherwise false:

    ::

        >>> mathtools.is_integer_pair(('some', 'text'))
        False

    Returns boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 2 and \
        all(isinstance(x, int) for x in expr)


# -*- encoding: utf-8 -*-
import re


uppercamelcase_regex = re.compile('^([A-Z,0-9]+[a-z,0-9]*)*$')

def is_upper_camel_case(expr):
    r'''Is true when `expr` is a string and is uppercamelcase.

    ..  container:: example

        ::

            >>> stringtools.is_upper_camel_case('FooBar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_upper_camel_case('fooBar')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(uppercamelcase_regex.match(expr))


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class TupletSpellingSpecifier(AbjadValueObject):
    r'''Tuplet spelling specifier.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_avoid_dots',
        '_is_diminution',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        avoid_dots=False,
        is_diminution=True,
        ):
        assert isinstance(avoid_dots, bool), avoid_dots
        assert isinstance(is_diminution, bool), is_diminution
        self._avoid_dots = avoid_dots
        self._is_diminution = is_diminution

    ### PUBLIC PROPERTIES ###

    @property
    def avoid_dots(self):
        r'''Is true when tuplet spelling should avoid dotted rhythmic values.
        Otherwise false.

        Defaults to false.

        Returns boolean.
        '''
        return self._avoid_dots

    @property
    def is_diminution(self):
        r'''Is true when tuplet should be spelled as diminution. Otherwise
        false.

        Defaults to true.

        Returns boolean.
        '''
        return self._is_diminution